import json
import os
import platform
import textwrap

import pytest

from conan.internal.util.files import load, rmdir
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.tools.cmake.presets import load_cmake_presets


@pytest.mark.tool("cmake", "3.23")
def test_cmake_presets_multiple_settings_single_config():
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_exe -d name=hello -d version=0.1")
    settings_layout = '-c tools.cmake.cmake_layout:build_folder_vars=' \
                      '\'["settings.compiler", "settings.compiler.version", ' \
                      '   "settings.compiler.cppstd"]\''

    user_presets_path = os.path.join(client.current_folder, "CMakeUserPresets.json")

    # Check that all generated names are expected, both in the layout and in the Presets
    settings = "-s compiler=apple-clang -s compiler.libcxx=libc++ " \
               "-s compiler.version=12.0 -s compiler.cppstd=gnu17"
    client.run("install . {} {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, "build", "apple-clang-12.0-gnu17",
                                       "Release", "generators"))
    assert os.path.exists(user_presets_path)
    user_presets = json.loads(load(user_presets_path))
    assert len(user_presets["include"]) == 1
    presets = json.loads(client.load(user_presets["include"][0]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 1
    assert len(presets["testPresets"]) == 1
    assert presets["configurePresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["buildPresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["testPresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["testPresets"][0]["configurePreset"] == "conan-apple-clang-12.0-gnu17-release"

    # If we create the "Debug" one, it will be appended
    client.run("install . {} -s build_type=Debug {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, "build", "apple-clang-12.0-gnu17",
                                       "Release", "generators"))
    assert os.path.exists(user_presets_path)
    user_presets = json.loads(load(user_presets_path))
    assert len(user_presets["include"]) == 2
    presets = json.loads(client.load(user_presets["include"][0]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 1
    assert len(presets["testPresets"]) == 1
    assert presets["configurePresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["buildPresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["testPresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-release"
    assert presets["testPresets"][0]["configurePreset"] == "conan-apple-clang-12.0-gnu17-release"

    presets = json.loads(client.load(user_presets["include"][1]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 1
    assert len(presets["testPresets"]) == 1
    assert presets["configurePresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-debug"
    assert presets["buildPresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-debug"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-apple-clang-12.0-gnu17-debug"
    assert presets["testPresets"][0]["name"] == "conan-apple-clang-12.0-gnu17-debug"
    assert presets["testPresets"][0]["configurePreset"] == "conan-apple-clang-12.0-gnu17-debug"

    # But If we change, for example, the cppstd and the compiler version, the toolchain
    # and presets will be different, but it will be appended to the UserPresets.json
    settings = "-s compiler=apple-clang -s compiler.libcxx=libc++ " \
               "-s compiler.version=13 -s compiler.cppstd=gnu20"
    client.run("install . {} {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, "build", "apple-clang-13-gnu20",
                                       "Release", "generators"))
    assert os.path.exists(user_presets_path)
    user_presets = json.loads(load(user_presets_path))
    # The [0] is the apple-clang 12 the [1] is the apple-clang 13
    assert len(user_presets["include"]) == 3
    presets = json.loads(client.load(user_presets["include"][2]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 1
    assert len(presets["testPresets"]) == 1
    assert presets["configurePresets"][0]["name"] == "conan-apple-clang-13-gnu20-release"
    assert presets["buildPresets"][0]["name"] == "conan-apple-clang-13-gnu20-release"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-apple-clang-13-gnu20-release"
    assert presets["testPresets"][0]["name"] == "conan-apple-clang-13-gnu20-release"
    assert presets["testPresets"][0]["configurePreset"] == "conan-apple-clang-13-gnu20-release"

    # We can build with cmake manually
    if platform.system() == "Darwin":
        client.run_command("cmake . --preset conan-apple-clang-12.0-gnu17-release")
        client.run_command("cmake --build --preset conan-apple-clang-12.0-gnu17-release")
        client.run_command("ctest --preset conan-apple-clang-12.0-gnu17-release")
        client.run_command("./build/apple-clang-12.0-gnu17/Release/hello")
        assert "Hello World Release!" in client.out
        assert "__cplusplus2017" in client.out

        client.run_command("cmake . --preset conan-apple-clang-12.0-gnu17-debug")
        client.run_command("cmake --build --preset conan-apple-clang-12.0-gnu17-debug")
        client.run_command("ctest --preset conan-apple-clang-12.0-gnu17-debug")
        client.run_command("./build/apple-clang-12.0-gnu17/Debug/hello")
        assert "Hello World Debug!" in client.out
        assert "__cplusplus2017" in client.out

        client.run_command("cmake . --preset conan-apple-clang-13-gnu20-release")
        client.run_command("cmake --build --preset conan-apple-clang-13-gnu20-release")
        client.run_command("ctest --preset conan-apple-clang-13-gnu20-release")
        client.run_command("./build/apple-clang-13-gnu20/Release/hello")
        assert "Hello World Release!" in client.out
        assert "__cplusplus2020" in client.out


@pytest.mark.parametrize("multiconfig", [True, False])
def test_cmake_presets_duplicated_install(multiconfig):
    # https://github.com/conan-io/conan/issues/11409
    """Only failed when using a multiconfig generator"""
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_exe -d name=hello -d version=0.1")
    settings = '-s compiler=gcc -s compiler.version=5 -s compiler.libcxx=libstdc++11 ' \
               '-c tools.cmake.cmake_layout:build_folder_vars=' \
               '\'["settings.compiler", "settings.compiler.version"]\' '
    if multiconfig:
        settings += '-c tools.cmake.cmaketoolchain:generator="Multi-Config"'
    client.run("install . {}".format(settings))
    client.run("install . {}".format(settings))
    if multiconfig:
        presets_path = os.path.join(client.current_folder, "build", "gcc-5", "generators",
                                    "CMakePresets.json")
    else:
        presets_path = os.path.join(client.current_folder, "build", "gcc-5", "Release", "generators",
                                    "CMakePresets.json")
    assert os.path.exists(presets_path)
    contents = json.loads(load(presets_path))
    assert len(contents["buildPresets"]) == 1
    assert len(contents["testPresets"]) == 1


def test_remove_missing_presets():
    # https://github.com/conan-io/conan/issues/11413
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_exe -d name=hello -d version=0.1")
    settings = '-s compiler=gcc -s compiler.version=5 -s compiler.libcxx=libstdc++11 ' \
               '-c tools.cmake.cmake_layout:build_folder_vars=' \
               '\'["settings.compiler", "settings.compiler.version"]\' '
    client.run("install . {}".format(settings))
    client.run("install . {} -s compiler.version=6".format(settings))

    presets_path_5 = os.path.join(client.current_folder, "build", "gcc-5")
    assert os.path.exists(presets_path_5)

    presets_path_6 = os.path.join(client.current_folder, "build", "gcc-6")
    assert os.path.exists(presets_path_6)

    rmdir(presets_path_5)

    # If we generate another configuration, the missing one (removed) for gcc-5 is not included
    client.run("install . {} -s compiler.version=11".format(settings))

    user_presets_path = os.path.join(client.current_folder, "CMakeUserPresets.json")
    assert os.path.exists(user_presets_path)

    contents = json.loads(load(user_presets_path))
    assert len(contents["include"]) == 2
    assert "gcc-6" in contents["include"][0]
    assert "gcc-11" in contents["include"][1]


@pytest.mark.tool("cmake", "3.23")
def test_cmake_presets_options_single_config():
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_lib -d name=hello -d version=0.1")
    conf_layout = '-c tools.cmake.cmake_layout:build_folder_vars=\'["settings.compiler",' \
                  '"settings.build_type", "options.shared"]\''

    default_compiler = {"Darwin": "apple-clang",
                        "Windows": "msvc",
                        "Linux": "gcc"}.get(platform.system())

    for shared in (True, False):
        client.run("install . {} -o shared={}".format(conf_layout, shared))
        shared_str = "shared" if shared else "static"
        assert os.path.exists(os.path.join(client.current_folder,
                                           "build", f"{default_compiler}-release-{shared_str}",
                                           "generators"))

    client.run("install . {}".format(conf_layout))
    assert os.path.exists(os.path.join(client.current_folder,
                                       "build", "{}-release-static".format(default_compiler),
                                       "generators"))

    user_presets_path = os.path.join(client.current_folder, "CMakeUserPresets.json")
    assert os.path.exists(user_presets_path)

    # We can build with cmake manually
    if platform.system() == "Darwin":
        for shared in (True, False):
            shared_str = "shared" if shared else "static"
            client.run_command(f"cmake . --preset conan-apple-clang-release-{shared_str}")
            client.run_command(f"cmake --build --preset conan-apple-clang-release-{shared_str}")
            client.run_command(f"ctest --preset conan-apple-clang-release-{shared_str}")
            the_lib = "libhello.a" if not shared else "libhello.dylib"
            path = os.path.join(client.current_folder,
                                "build", "apple-clang-release-{}".format(shared_str), the_lib)
            assert os.path.exists(path)


@pytest.mark.tool("cmake", "3.23")
def test_cmake_presets_with_conanfile_txt():
    c = TestClient()

    c.run("new cmake_exe -d name=foo -d version=1.0")
    os.unlink(os.path.join(c.current_folder, "conanfile.py"))
    c.save({"conanfile.txt": textwrap.dedent("""
        [generators]
        CMakeToolchain

        [layout]
        cmake_layout
        """)})

    c.run("install .")
    c.run("install . -s build_type=Debug")

    if platform.system() != "Windows":
        c.run_command("cmake --preset conan-debug")
        c.run_command("cmake --build --preset conan-debug")
        c.run_command("ctest --preset conan-debug")
        c.run_command("./build/Debug/foo")
    else:
        c.run_command("cmake --preset conan-default")
        # this second called used to fail
        # https://github.com/conan-io/conan/issues/13792, CMake ignoring toolchain architecture
        # and toolset
        c.run_command("cmake --preset conan-default")
        c.run_command("cmake --build --preset conan-debug")
        c.run_command("ctest --preset conan-debug")
        c.run_command("build\\Debug\\foo")

    assert "Hello World Debug!" in c.out

    if platform.system() != "Windows":
        c.run_command("cmake --preset conan-release")
        c.run_command("cmake --build --preset conan-release")
        c.run_command("ctest --preset conan-release")
        c.run_command("./build/Release/foo")
    else:
        c.run_command("cmake --build --preset conan-release")
        c.run_command("ctest --preset conan-release")
        c.run_command("build\\Release\\foo")

    assert "Hello World Release!" in c.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
@pytest.mark.tool("ninja")
@pytest.mark.tool("cmake", "3.23")
def test_cmake_presets_with_conanfile_txt_ninja():
    c = TestClient()

    c.run("new cmake_exe -d name=foo -d version=1.0")
    os.unlink(os.path.join(c.current_folder, "conanfile.py"))
    c.save({"conanfile.txt": textwrap.dedent("""
        [generators]
        CMakeToolchain

        [layout]
        cmake_layout
        """)})

    conf = "-c tools.cmake.cmaketoolchain:generator=Ninja"
    c.run(f"install . {conf}")
    c.run(f"install . -s build_type=Debug {conf}")

    c.run_command("build\\Release\\generators\\conanbuild.bat && cmake --preset conan-release")
    c.run_command("build\\Release\\generators\\conanbuild.bat && cmake --preset conan-release")
    c.run_command("build\\Release\\generators\\conanbuild.bat "
                  "&& cmake --build --preset conan-release")
    c.run_command("build\\Release\\generators\\conanbuild.bat && ctest --preset conan-release")
    c.run_command("build\\Release\\foo")

    assert "Hello World Release!" in c.out


def test_cmake_toolchain_custom_toolchain():
    client = TestClient(path_with_spaces=False)
    conanfile = GenConanfile().with_settings("os", "compiler", "build_type", "arch").\
        with_generator("CMakeToolchain")
    client.save_home({"global.conf": "tools.cmake.cmaketoolchain:toolchain_file=mytoolchain.cmake"})

    client.save({"conanfile.py": conanfile})
    client.run("install .")
    assert not os.path.exists(os.path.join(client.current_folder, "conan_toolchain.cmake"))
    presets = load_cmake_presets(client.current_folder)
    assert "mytoolchain.cmake" in presets["configurePresets"][0]["toolchainFile"]
    assert "binaryDir" in presets["configurePresets"][0]


@pytest.mark.skipif(platform.system() != "Darwin",
                    reason="Single config test, Linux CI still without 3.23")
@pytest.mark.tool("cmake", "3.23")
@pytest.mark.parametrize("existing_user_presets", [None, "user_provided", "conan_generated"])
def test_cmake_user_presets_load(existing_user_presets):
    """
    Test if the CMakeUserPresets.cmake is generated and use CMake to use it to verify the right
    syntax of generated CMakeUserPresets.cmake and CMakePresets.cmake. If the user already provided
    a CMakeUserPresets.cmake, leave the file untouched, and only generate or modify the file if
    the `conan` object exists in the `vendor` field.
    """
    t = TestClient()
    t.run("new -d name=mylib -d version=1.0 -f cmake_lib")
    t.run("create . -s:h build_type=Release")
    t.run("create . -s:h build_type=Debug")

    consumer = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class Consumer(ConanFile):

            settings = "build_type", "os", "arch", "compiler"
            requires = "mylib/1.0"
            generators = "CMakeToolchain", "CMakeDeps"

            def layout(self):
                cmake_layout(self)

    """)

    cmakelist = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.1)
        project(PackageTest NONE)
        find_package(mylib REQUIRED CONFIG)
        """)

    user_presets = None
    if existing_user_presets == "user_provided":
        user_presets = "{}"
    elif existing_user_presets == "conan_generated":
        user_presets = '{ "vendor": {"conan": {} } }'

    files_to_save = {"conanfile.py": consumer, "CMakeLists.txt": cmakelist}

    if user_presets:
        files_to_save['CMakeUserPresets.json'] = user_presets
    t.save(files_to_save, clean_first=True)
    t.run("install . -s:h build_type=Debug -g CMakeToolchain")
    t.run("install . -s:h build_type=Release -g CMakeToolchain")

    user_presets_path = os.path.join(t.current_folder, "CMakeUserPresets.json")
    assert os.path.exists(user_presets_path)

    user_presets_data = json.loads(load(user_presets_path))
    if existing_user_presets == "user_provided":
        assert not user_presets_data
    else:
        assert "include" in user_presets_data.keys()

    if existing_user_presets is None:
        t.run_command("cmake . --preset conan-release")
        assert 'CMAKE_BUILD_TYPE="Release"' in t.out
        t.run_command("cmake . --preset conan-debug")
        assert 'CMAKE_BUILD_TYPE="Debug"' in t.out


@pytest.mark.tool("cmake", "3.23")
@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
def test_cmake_presets_multiple_settings_multi_config():
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_exe -d name=hello -d version=0.1")
    settings_layout = '-c tools.cmake.cmake_layout:build_folder_vars=' \
                      '\'["settings.compiler.runtime", "settings.compiler.cppstd"]\''

    user_presets_path = os.path.join(client.current_folder, "CMakeUserPresets.json")

    # Check that all generated names are expected, both in the layout and in the Presets
    settings = "-s compiler=msvc -s compiler.version=191 -s compiler.runtime=dynamic " \
               "-s compiler.cppstd=14"
    client.run("install . {} {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, "build", "dynamic-14", "generators"))
    assert os.path.exists(user_presets_path)
    user_presets = json.loads(load(user_presets_path))
    assert len(user_presets["include"]) == 1
    presets = json.loads(client.load(user_presets["include"][0]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 1
    assert len(presets["testPresets"]) == 1
    assert presets["configurePresets"][0]["name"] == "conan-dynamic-14"
    assert presets["buildPresets"][0]["name"] == "conan-dynamic-14-release"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-dynamic-14"
    assert presets["testPresets"][0]["name"] == "conan-dynamic-14-release"
    assert presets["testPresets"][0]["configurePreset"] == "conan-dynamic-14"

    # If we create the "Debug" one, it has the same toolchain and preset file, that is
    # always multiconfig
    client.run("install . {} -s build_type=Debug {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, "build", "dynamic-14", "generators"))
    assert os.path.exists(user_presets_path)
    user_presets = json.loads(load(user_presets_path))
    assert len(user_presets["include"]) == 1
    presets = json.loads(client.load(user_presets["include"][0]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 2
    assert len(presets["testPresets"]) == 2
    assert presets["configurePresets"][0]["name"] == "conan-dynamic-14"
    assert presets["buildPresets"][0]["name"] == "conan-dynamic-14-release"
    assert presets["buildPresets"][1]["name"] == "conan-dynamic-14-debug"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-dynamic-14"
    assert presets["buildPresets"][1]["configurePreset"] == "conan-dynamic-14"
    assert presets["testPresets"][0]["name"] == "conan-dynamic-14-release"
    assert presets["testPresets"][1]["name"] == "conan-dynamic-14-debug"
    assert presets["testPresets"][0]["configurePreset"] == "conan-dynamic-14"
    assert presets["testPresets"][1]["configurePreset"] == "conan-dynamic-14"

    # But If we change, for example, the cppstd and the compiler version, the toolchain
    # and presets will be different, but it will be appended to the UserPresets.json
    settings = "-s compiler=msvc -s compiler.version=191 -s compiler.runtime=static " \
               "-s compiler.cppstd=17"
    client.run("install . {} {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, "build", "static-17", "generators"))
    assert os.path.exists(user_presets_path)
    user_presets = json.loads(load(user_presets_path))
    # The [0] is the msvc dynamic/14 the [1] is the static/17
    assert len(user_presets["include"]) == 2
    presets = json.loads(client.load(user_presets["include"][1]))
    assert len(presets["configurePresets"]) == 1
    assert len(presets["buildPresets"]) == 1
    assert len(presets["testPresets"]) == 1
    assert presets["configurePresets"][0]["name"] == "conan-static-17"
    assert presets["buildPresets"][0]["name"] == "conan-static-17-release"
    assert presets["buildPresets"][0]["configurePreset"] == "conan-static-17"
    assert presets["testPresets"][0]["name"] == "conan-static-17-release"
    assert presets["testPresets"][0]["configurePreset"] == "conan-static-17"

    # We can build with cmake manually
    client.run_command("cmake . --preset conan-dynamic-14")

    client.run_command("cmake --build --preset conan-dynamic-14-release")
    client.run_command("ctest --preset conan-dynamic-14-release")
    client.run_command("build\\dynamic-14\\Release\\hello")
    assert "Hello World Release!" in client.out
    assert "MSVC_LANG2014" in client.out

    client.run_command("cmake --build --preset conan-dynamic-14-debug")
    client.run_command("ctest --preset conan-dynamic-14-debug")
    client.run_command("build\\dynamic-14\\Debug\\hello")
    assert "Hello World Debug!" in client.out
    assert "MSVC_LANG2014" in client.out

    client.run_command("cmake . --preset conan-static-17")

    client.run_command("cmake --build --preset conan-static-17-release")
    client.run_command("ctest --preset conan-static-17-release")
    client.run_command("build\\static-17\\Release\\hello")
    assert "Hello World Release!" in client.out
    assert "MSVC_LANG2017" in client.out


@pytest.mark.tool("cmake", "3.23")
@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
# Test both with a local folder and an absolute folder
@pytest.mark.parametrize("build", ["mybuild", "temp"])
def test_cmake_presets_build_folder(build):
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_exe -d name=hello -d version=0.1")

    build = temp_folder() if build == "temp" else build
    settings_layout = f' -c tools.cmake.cmake_layout:build_folder="{build}" '\
                      '-c tools.cmake.cmake_layout:build_folder_vars=' \
                      '\'["settings.compiler.runtime", "settings.compiler.cppstd"]\''
    # But If we change, for example, the cppstd and the compiler version, the toolchain
    # and presets will be different, but it will be appended to the UserPresets.json
    settings = "-s compiler=msvc -s compiler.version=191 -s compiler.runtime=static " \
               "-s compiler.cppstd=17"
    client.run("install . {} {}".format(settings, settings_layout))
    assert os.path.exists(os.path.join(client.current_folder, build, "static-17", "generators"))

    client.run_command("cmake . --preset conan-static-17")
    client.run_command("cmake --build --preset conan-static-17-release")
    client.run_command("ctest --preset conan-static-17-release")
    client.run_command(f"{build}\\static-17\\Release\\hello")
    assert "Hello World Release!" in client.out
    assert "MSVC_LANG2017" in client.out


@pytest.mark.tool("cmake", "3.23")
class TestEnvironmentInPresets:
    @pytest.fixture(scope="class")
    def _init_client(self):
        c = TestClient()
        tool = textwrap.dedent(r"""
            import os
            from conan import ConanFile
            from conan.tools.files import chdir, save
            class Tool(ConanFile):
                version = "0.1"
                settings = "os", "compiler", "arch", "build_type"
                def package(self):
                    with chdir(self, self.package_folder):
                        v = f"{{self.name}}/{{self.version}}"
                        save(self, f"bin/{{self.name}}.bat", f"@echo off\necho running: {{v}}")
                        save(self, f"bin/{{self.name}}.sh", f"echo running: {{v}}")
                        os.chmod(f"bin/{{self.name}}.sh", 0o777)
                def package_info(self):
                    self.buildenv_info.define("MY_BUILD_VAR", "MY_BUILDVAR_VALUE")
                    {}
            """)

        consumer = textwrap.dedent("""
            [tool_requires]
            mytool/0.1
            [test_requires]
            mytesttool/0.1
            [layout]
            cmake_layout
        """)

        test_env = textwrap.dedent("""
            #include <cstdlib>
            int main() {
                return std::getenv("MY_RUNVAR") ? 0 : 1;
            }
        """)

        cmakelists = textwrap.dedent("""
            set(CMAKE_CXX_COMPILER_WORKS 1)
            set(CMAKE_CXX_ABI_COMPILED 1)
            cmake_minimum_required(VERSION 3.15)
            project(MyProject CXX)

            if(WIN32)
                set(MYTOOL_SCRIPT "mytool.bat")
            else()
                set(MYTOOL_SCRIPT "mytool.sh")
            endif()

            add_custom_target(run_mytool COMMAND ${MYTOOL_SCRIPT})

            function(check_and_report_variable var_name)
                set(var_value $ENV{${var_name}})
                if (var_value)
                    message("${var_name}:${var_value}")
                else()
                    message("${var_name} NOT FOUND")
                endif()
            endfunction()

            check_and_report_variable("MY_BUILD_VAR")
            check_and_report_variable("MY_RUNVAR")
            check_and_report_variable("MY_ENV_VAR")

            enable_testing()
            add_executable(test_env test_env.cpp)
            add_test(NAME TestRunEnv COMMAND test_env)
        """)

        c.save({"tool.py": tool.format(""),
                "test_tool.py":
                    tool.format('self.runenv_info.define("MY_RUNVAR", "MY_RUNVAR_VALUE")'),
                "conanfile.txt": consumer,
                "CMakeLists.txt": cmakelists,
                "test_env.cpp": test_env})

        c.run("create tool.py --name=mytool")

        c.run("create test_tool.py --name=mytesttool")
        c.run("create test_tool.py --name=mytesttool -s build_type=Debug")

        yield c

    def test_add_env_to_presets(self, _init_client):
        c = _init_client

        # do a first conan install with env disabled just to test that the conf works
        c.run("install . -g CMakeToolchain -g CMakeDeps "
              "-c tools.cmake.cmaketoolchain:presets_environment=disabled")

        presets_path = os.path.join("build", "Release", "generators", "CMakePresets.json") \
            if platform.system() != "Windows" else os.path.join("build", "generators",
                                                                "CMakePresets.json")
        presets = json.loads(c.load(presets_path))

        assert presets["configurePresets"][0].get("env") is None

        c.run("install . -g CMakeToolchain -g CMakeDeps")
        c.run("install . -g CMakeToolchain -g CMakeDeps -s:h build_type=Debug")

        # test that the buildenv is correctly injected to configure and build steps
        # that the runenv is not injected to configure, but it is when running tests

        preset = "conan-default" if platform.system() == "Windows" else "conan-release"

        c.run_command(f"cmake --preset {preset}")
        assert "MY_BUILD_VAR:MY_BUILDVAR_VALUE" in c.out
        assert "MY_RUNVAR NOT FOUND" in c.out
        c.run_command("cmake --build --preset conan-release --target run_mytool --target test_env")
        assert "running: mytool/0.1" in c.out

        c.run_command("ctest --preset conan-release")
        assert "tests passed" in c.out

        if platform.system() != "Windows":
            c.run_command("cmake --preset conan-debug")
            assert "MY_BUILD_VAR:MY_BUILDVAR_VALUE" in c.out
            assert "MY_RUNVAR NOT FOUND" in c.out

        c.run_command("cmake --build --preset conan-debug --target run_mytool --target test_env")
        assert "running: mytool/0.1" in c.out

        c.run_command("ctest --preset conan-debug")
        assert "tests passed" in c.out

    def test_add_generate_env_to_presets(self, _init_client):
        c = _init_client

        consumer = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.cmake import cmake_layout, CMakeToolchain, CMakeDeps
            from conan.tools.env import VirtualBuildEnv, VirtualRunEnv, Environment

            class mypkgRecipe(ConanFile):
                name = "mypkg"
                version = "1.0"
                settings = "os", "compiler", "build_type", "arch"
                tool_requires = "mytool/0.1"
                test_requires = "mytesttool/0.1"
                def layout(self):
                    cmake_layout(self)

                def generate(self):

                    buildenv = VirtualBuildEnv(self)
                    buildenv.environment().define("MY_BUILD_VAR", "MY_BUILDVAR_VALUE_OVERRIDEN")
                    buildenv.generate()

                    runenv = VirtualRunEnv(self)
                    runenv.environment().define("MY_RUN_VAR", "MY_RUNVAR_SET_IN_GENERATE")
                    runenv.generate()

                    env = Environment()
                    env.define("MY_ENV_VAR", "MY_ENV_VAR_VALUE")
                    env = env.vars(self)
                    env.save_script("other_env")

                    deps = CMakeDeps(self)
                    deps.generate()
                    tc = CMakeToolchain(self)
                    tc.presets_build_environment = buildenv.environment().compose_env(env)
                    tc.presets_run_environment = runenv.environment()
                    tc.generate()
        """)

        test_env = textwrap.dedent("""
            #include <string>
            int main() {
                return std::string(std::getenv("MY_RUNVAR"))=="MY_RUNVAR_SET_IN_GENERATE";
            }
        """)

        c.save({"conanfile.py": consumer,
                "test_env.cpp": test_env})

        c.run("install conanfile.py")

        preset = "conan-default" if platform.system() == "Windows" else "conan-release"

        c.run_command(f"cmake --preset {preset}")
        assert "MY_BUILD_VAR:MY_BUILDVAR_VALUE_OVERRIDEN" in c.out
        assert "MY_ENV_VAR:MY_ENV_VAR_VALUE" in c.out

        c.run_command("cmake --build --preset conan-release --target run_mytool --target test_env")
        assert "running: mytool/0.1" in c.out

        c.run_command(f"ctest --preset conan-release")
        assert "tests passed" in c.out



import pytest

from conan.tools.build import supported_cstd, check_min_cstd
from conan.errors import ConanException
from conan.test.utils.mocks import MockSettings, ConanFileMock


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("clang", "2.1", ['99', 'gnu99']),
    ("clang", "5", ['99', 'gnu99', '11', 'gnu11']),
    ("clang", "17", ['99', 'gnu99', '11', 'gnu11', "17", "gnu17"]),
    ("clang", "18", ['99', 'gnu99', '11', 'gnu11',  "17", "gnu17", "23", "gnu23"])
])
def test_supported_cstd_clang(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = ConanFileMock(settings)
    sot = supported_cstd(conanfile)
    assert sot == values


def test_supported_cstd_with_specific_values():
    settings = MockSettings({})
    conanfile = ConanFileMock(settings)
    sot = supported_cstd(conanfile, "clang", "5")
    assert sot == ['99', 'gnu99', '11', 'gnu11']


def test_supported_cstd_error():
    settings = MockSettings({})
    conanfile = ConanFileMock(settings)
    with pytest.raises(ConanException) as exc:
        supported_cstd(conanfile)
    assert "Called supported_cstd with no compiler or no compiler.version" in str(exc)


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("gcc", "4.6", ['99', 'gnu99']),
    ("gcc", "5", ['99', 'gnu99', '11', 'gnu11']),
    ("gcc", "13", ["99", "gnu99", "11", "gnu11", "17", "gnu17"]),
    ("gcc", "14", ["99", "gnu99", "11", "gnu11", "17", "gnu17", "23", "gnu23"])
])
def test_supported_cppstd_gcc(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = ConanFileMock(settings)
    sot = supported_cstd(conanfile)
    assert sot == values


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("msvc", "191", []),
    ("msvc", "193", ['11', '17']),
])
def test_supported_cstd_msvc(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = ConanFileMock(settings)
    sot = supported_cstd(conanfile)
    assert sot == values


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("emcc", "3", ["99", "gnu99", "11", "gnu11", "17", "gnu17"]),
    ("emcc", "3.1", ["99", "gnu99", "11", "gnu11", "17", "gnu17", "23", "gnu23"]),
    ("emcc", "3.3", ["99", "gnu99", "11", "gnu11", "17", "gnu17", "23", "gnu23"]),
    ("emcc", "4.1", ["99", "gnu99", "11", "gnu11", "17", "gnu17", "23", "gnu23"]),
])
def test_supported_cstd_emcc(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = ConanFileMock(settings)
    sot = supported_cstd(conanfile)
    assert sot == values


def test_check_cstd_type():
    """ cppstd must be a number
    """
    conanfile = ConanFileMock(MockSettings({}))
    with pytest.raises(ConanException) as exc:
        check_min_cstd(conanfile, "gnu17", False)

    assert "cstd parameter must be a number", str(exc)



import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestUserChannelTestPackage:

    def test(self):
        # https://github.com/conan-io/conan/issues/2501
        client = TestClient(light=True)
        conanfile = """from conan import ConanFile
class SayConan(ConanFile):
    pass
"""
        test = """from conan import ConanFile
class SayConan(ConanFile):
    def requirements(self):
        self.output.info("USER: %s!!" % self.user)
        self.output.info("CHANNEL: %s!!" % self.channel)
        self.requires(self.tested_reference_str)

    def test(self):
        pass
"""

        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": test})
        client.run("create . --name=pkg --version=0.1 --user=conan --channel=testing")
        assert "pkg/0.1@conan/testing (test package): USER: conan!!" in client.out
        assert "pkg/0.1@conan/testing (test package): CHANNEL: testing!!" in client.out


class TestSameUserChannel:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.client = TestClient(light=True)
        conanfile = """
from conan import ConanFile

class SayConan(ConanFile):
    name = "say"
    version = "0.1"
    build_policy = "missing"

    def build(self):
        self.output.info("Building %s/%s")
"""
        for user, channel in ("lasote", "stable"), ("other", "testing"):
            self.client.save({"conanfile.py": conanfile % (user, channel)})
            self.client.run(f"export . --user={user} --channel={channel}")

        self.conanfile = """
from conan import ConanFile

class HelloConan(ConanFile):
    name = "hello"
    version = "0.1"
    build_policy = "missing"

    def requirements(self):
        user_channel = "{}/{}".format(self.user, self.channel) if self.user else ""
        self.requires("say/0.1@{}".format(user_channel))

    def build(self):
        self.output.info("Building %s/%s" % (self.user, self.channel) )
"""

        self.test_conanfile = str(GenConanfile().with_test("pass"))
        self.client.save({"conanfile.py": self.conanfile,
                          "test/conanfile.py": self.test_conanfile})

    def test_create(self):
        self.client.run("create . --user=lasote --channel=stable")
        assert "say/0.1@lasote/stable: Building lasote/stable" in self.client.out
        assert "hello/0.1@lasote/stable: Building lasote/stable" in self.client.out
        assert "other/testing" not in self.client.out

        self.client.save({"conanfile.py": self.conanfile,
                          "test/conanfile.py": self.test_conanfile.replace("lasote/stable",
                                                                           "other/testing")})
        self.client.run("create . --user=other --channel=testing")
        assert "say/0.1@other/testing: Building other/testing" in self.client.out
        assert "hello/0.1@other/testing: Building other/testing" in self.client.out
        assert "lasote/stable" not in self.client.out

    def test_local_commands(self):
        self.client.run("install .", assert_error=True)
        assert "ERROR: Package 'say/0.1' not resolved: No remote defined" in self.client.out

        self.client.run("install . --user=lasote --channel=stable")
        assert "say/0.1@lasote/stable: Building lasote/stable" in self.client.out
        assert "other/testing" not in self.client.out

        self.client.run("install . --user=other --channel=testing")
        assert "say/0.1@other/testing: Building other/testing" in self.client.out
        assert "lasote/stable" not in self.client.out

        # Now use the default_ methods to declare user and channel
        self.client = TestClient(light=True)
        conanfile = """
from conan import ConanFile

class SayConan(ConanFile):
    name = "say"
    version = "0.1"
    build_policy = "missing"
    user = "userfoo"

    def build(self):
        self.output.info("Building %s/%s" % (self.user, self.channel) )

    @property
    def channel(self):
        return "channelbar"
"""
        self.client.save({"conanfile.py": conanfile})
        self.client.run("install .")
        self.client.run("build .")
        assert "Building userfoo/channelbar" in self.client.out


class TestBuildRequireUserChannel:
    def test(self):
        # https://github.com/conan-io/conan/issues/2254
        client = TestClient(light=True)
        conanfile = """
from conan import ConanFile

class SayConan(ConanFile):
    def build_requirements(self):
        self.output.info("MYUSER: %s" % self.user)
        self.output.info("MYCHANNEL: %s" % self.channel)
"""
        client.save({"conanfile.py": conanfile})
        client.run("install . --user=myuser --channel=mychannel")
        assert "MYUSER: myuser" in client.out
        assert "MYCHANNEL: mychannel" in client.out



import json
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_user_overrides():
    """ Show that it is possible to add things to lockfiles, to pre-lock things explicitly from
    user side
    """
    c = TestClient(light=True)
    c.save({"math/conanfile.py": GenConanfile("math"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("math/[*]"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_requires("engine/[*]")})

    c.run("export math --version=1.0")
    c.run("export math --version=1.1")
    c.run("export math --version=1.2")
    c.run("export engine")

    c.run("graph info game")
    assert "math/1.2" in c.out
    assert "math/1.0" not in c.out

    c.run("lock add --requires=math/1.0 --requires=unrelated/2.0")
    c.run("graph info game --lockfile=conan.lock --lockfile-out=new.lock --lockfile-partial")
    assert "math/1.0" in c.out
    assert "math/1.2" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert "math/1.0#8e1a7a5ce869d8c54ae3d33468fd657" in new_lock

    # Repeat for 1.1
    c.run("lock add --requires=math/1.1 --requires=unrelated/2.0")
    c.run("graph info game --lockfile=conan.lock --lockfile-partial --lockfile-out=new.lock")
    assert "math/1.1" in c.out
    assert "math/1.0" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert "math/1.1#8e1a7a5ce869d8c54ae3d33468fd657" in new_lock


def test_user_build_overrides():
    """ Test that it is possible to lock also build-requries
    """
    c = TestClient(light=True)
    c.save({"cmake/conanfile.py": GenConanfile("cmake"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_build_requires("cmake/[*]")})

    c.run("export cmake --version=1.0")
    c.run("export cmake --version=1.1")
    c.run("export cmake --version=1.2")

    c.run("graph info engine")
    assert "cmake/1.2" in c.out
    assert "cmake/1.0" not in c.out

    c.run("lock add --build-requires=cmake/1.0")
    c.run("graph info engine --lockfile=conan.lock --lockfile-out=new.lock --lockfile-partial")
    assert "cmake/1.0" in c.out
    assert "cmake/1.2" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert "cmake/1.0" in new_lock

    # Repeat for 1.1
    c.run("lock add --build-requires=cmake/1.1 --lockfile-out=conan.lock")
    c.run("graph info engine --lockfile=conan.lock --lockfile-out=new.lock --lockfile-partial")
    assert "cmake/1.1" in c.out
    assert "cmake/1.0" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert "cmake/1.1" in new_lock


def test_user_python_overrides():
    """ Test that it is possible to lock also python-requries
    """
    c = TestClient(light=True)
    c.save({"pytool/conanfile.py": GenConanfile("pytool"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_python_requires("pytool/[*]")})

    c.run("export pytool --version=1.0")
    c.run("export pytool --version=1.1")
    c.run("export pytool --version=1.2")

    c.run("graph info engine")
    assert "pytool/1.2" in c.out
    assert "pytool/1.0" not in c.out

    c.run("lock add --python-requires=pytool/1.0 --lockfile-out=conan.lock")
    c.run("graph info engine --lockfile=conan.lock --lockfile-out=new.lock")
    assert "pytool/1.0" in c.out
    assert "pytool/1.2" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert "pytool/1.0" in new_lock

    # Repeat for 1.1
    c.run("lock add --python-requires=pytool/1.1 --lockfile-out=conan.lock")
    c.run("graph info engine --lockfile=conan.lock --lockfile-out=new.lock")
    assert "pytool/1.1" in c.out
    assert "pytool/1.0" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert "pytool/1.1" in new_lock


def test_config_overrides():
    """ Test that it is possible to lock also config-requires
    """
    c = TestClient(light=True)
    c.run("lock add --config-requires=config/1.0")
    assert json.loads(c.load("conan.lock"))["config_requires"] == ["config/1.0"]
    c.run("lock remove --config-requires=config/1.0")
    assert json.loads(c.load("conan.lock"))["config_requires"] == []


def test_add_revisions():
    """ Is it possible to add revisions explicitly too
    """
    c = TestClient(light=True)
    c.save({"math/conanfile.py": GenConanfile("math"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("math/[*]"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_requires("engine/[*]")})

    c.run("export math --version=1.0")
    rev0 = c.exported_recipe_revision()
    c.save({"math/conanfile.py": GenConanfile("math").with_build_msg("New rev1")})
    c.run("export math --version=1.0")
    rev1 = c.exported_recipe_revision()
    c.save({"math/conanfile.py": GenConanfile("math").with_build_msg("New rev2")})
    c.run("export math --version=1.0")
    rev2 = c.exported_recipe_revision()

    c.run("export engine")
    c.run("graph info game")
    assert f"math/1.0#{rev2}" in c.out
    assert f"math/1.0#{rev1}" not in c.out

    # without revision, it will resolve to latest
    c.run("lock add --requires=math/1.0 --requires=unrelated/2.0")
    c.run("graph info game --lockfile=conan.lock --lockfile-out=new.lock --lockfile-partial")
    assert f"math/1.0#{rev2}" in c.out
    assert f"math/1.0#{rev1}" not in c.out
    assert f"math/1.0#{rev0}" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert f"math/1.0#{rev2}" in new_lock
    assert f"math/1.0#{rev1}" not in new_lock
    assert f"math/1.0#{rev0}" not in c.out

    # with revision, it will resolve to that revision
    c.run(f"lock add --requires=math/1.0#{rev1} --requires=unrelated/2.0")
    c.run("graph info game --lockfile=conan.lock --lockfile-out=new.lock --lockfile-partial")
    assert f"math/1.0#{rev1}" in c.out
    assert f"math/1.0#{rev2}" not in c.out
    assert f"math/1.0#{rev0}" not in c.out
    # The resulting lockfile contains the full revision now
    new_lock = c.load("new.lock")
    assert f"math/1.0#{rev1}" in new_lock
    assert f"math/1.0#{rev2}" not in new_lock
    assert f"math/1.0#{rev0}" not in c.out


def test_add_multiple_revisions():
    """ What if we add multiple revisions, mix with and without revisions, with and without
    timestamps and it will not crash
    """
    c = TestClient(light=True)
    # without revision, it will resolve to latest
    c.run("lock add --requires=math/1.0#rev1")
    new_lock = c.load("conan.lock")
    assert "math/1.0#rev1" in new_lock

    c.run("lock add --requires=math/1.0#rev2")
    new_lock = json.loads(c.load("conan.lock"))
    assert ["math/1.0#rev2", "math/1.0#rev1"] == new_lock["requires"]
    c.run("lock add --requires=math/1.0#rev0")
    new_lock = json.loads(c.load("conan.lock"))
    assert ['math/1.0#rev2', 'math/1.0#rev1', 'math/1.0#rev0'] == new_lock["requires"]
    c.run("lock add --requires=math/1.0#revx%0.0")
    new_lock = json.loads(c.load("conan.lock"))
    assert ['math/1.0#revx%0.0', 'math/1.0#rev2', 'math/1.0#rev1', 'math/1.0#rev0'] == \
           new_lock["requires"]

    c.save({"conanfile.txt": ""})
    c.run("install . --lockfile=conan.lock")  # Just check that it doesn't crash
    c.run("install . --lockfile=conan.lock --lockfile-out=new.lock")
    new_lock = json.loads(c.load("conan.lock"))
    assert ['math/1.0#revx%0.0', 'math/1.0#rev2', 'math/1.0#rev1', 'math/1.0#rev0'] == \
           new_lock["requires"]

    # add without revision at all, will give us an error, as it doesn't make sense
    c.run("lock add --requires=math/1.0", assert_error=True)
    assert "Cannot add math/1.0 to lockfile, already exists" in c.out
    new_lock = json.loads(c.load("conan.lock"))
    assert ['math/1.0#revx%0.0', 'math/1.0#rev2', 'math/1.0#rev1', 'math/1.0#rev0'] == \
           new_lock["requires"]


def test_timestamps_are_updated():
    """ When ``conan lock add`` adds a revision with a timestamp, or without it, it will be
    updated in the lockfile-out to the resolved new timestamp
    """
    c = TestClient(light=True)
    c.save({"conanfile.txt": "[requires]\nmath/1.0",
            "math/conanfile.py": GenConanfile("math", "1.0")})
    c.run("create math")
    rev = c.exported_recipe_revision()
    # Create a new lockfile, wipe the previous
    c.run(f"lock add --lockfile=\"\" --requires=math/1.0#{rev}%0.123")
    c.run("install . --lockfile=conan.lock --lockfile-out=conan.lock")
    assert f" math/1.0#{rev} - Cache" in c.out
    new_lock = c.load("conan.lock")
    assert "%0.123" not in new_lock


def test_lock_add_error():
    # https://github.com/conan-io/conan/issues/14465
    c = TestClient(light=True)
    c.run(f"lock add --requires=math/1.0:pid1", assert_error=True)
    assert "ERROR: Invalid recipe reference 'math/1.0:pid1' is a package reference" in c.out


class TestLockRemove:
    @pytest.mark.parametrize("args, removed", [
        ("--requires=math/*", ["math"]),
        ("--requires=math/2.0", []),
        ("--build-requires=cmake/1.0", ["cmake"]),
        # Not valid ("--build-requires=*", ["cmake", "ninja"]),
        ("--build-requires=*/*", ["cmake", "ninja"]),  # But this is valid
        ("--python-requires=mytool/*", ["mytool"]),
        ("--python-requires=*tool/*", ["mytool", "othertool"]),
        # With version ranges
        ('--requires="math/[>=1.0 <2]"', ["math"]),
        ('--requires="math/[>1.0]"', []),
        ('--requires="*/[>=1.0 <2]"', ["math", "engine"])
    ])
    def test_lock_remove(self, args, removed):
        c = TestClient(light=True)
        lock = textwrap.dedent("""\
            {
                "version": "0.5",
                "requires": [
                    "math/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "math/1.0#12345%1702683584.3411012",
                    "engine/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ],
                "build_requires": [
                    "cmake/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "ninja/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ],
                "python_requires": [
                    "mytool/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "othertool/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ]
            }
            """)
        c.save({"conan.lock": lock})
        c.run(f"lock remove {args}")
        lock = c.load("conan.lock")
        for remove in removed:
            assert remove not in lock
        for pkg in {"math", "engine", "cmake", "ninja", "mytool", "othertool"}.difference(removed):
            assert pkg in lock

    @pytest.mark.parametrize("args, removed", [
        ("--requires=math/1.0#12345*", ["math/1.0#123456789abcdef"]),
        ("--requires=math/1.0#*", ["math/1.0#123456789abcdef",
                                   "math/1.0#85d927a4a067a531b1a9c7619522c015"]),
    ])
    def test_lock_remove_revisions(self, args, removed):
        c = TestClient(light=True)
        lock = textwrap.dedent("""\
            {
                "version": "0.5",
                "requires": [
                    "math/1.0#123456789abcdef%1702683584.3411012",
                    "math/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "engine/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ]
            }
            """)
        c.save({"conan.lock": lock})
        c.run(f"lock remove {args}")
        lock = c.load("conan.lock")
        for remove in removed:
            assert remove not in lock
        for pkg in {"math/1.0#123456789abcdef",
                    "math/1.0#85d927a4a067a531b1a9c7619522c015",
                    "engine/1.0#fd2b006646a54397c16a1478ac4111ac"}.difference(removed):
            assert pkg in lock

    @pytest.mark.parametrize("args, removed", [
        ("--requires=*/*@team", ["pkg/1.0@team"]),
        ("--requires=*/*@team*", ["pkg/1.0@team", "math/2.0@team/stable"]),
        ("--requires=*/*@user", ["math/1.0@user", "other/1.0@user"]),
        ("--requires=*/*@", ["engine/1.0"]),  # Remove those without user
        # with version ranges
        ("--requires=math/[*]@user", ["math/1.0@user"]),
        ("--requires=math/[*]@team*", ["math/2.0@team/stable"]),
    ])
    def test_lock_remove_user_channel(self, args, removed):
        c = TestClient(light=True)
        lock = textwrap.dedent("""\
            {
                "version": "0.5",
                "requires": [
                    "math/1.0@user#123456789abcdef%1702683584.3411012",
                    "math/2.0@team/stable#123456789abcdef%1702683584.3411012",
                    "other/1.0@user#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "pkg/1.0@team#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "engine/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ]
            }
            """)
        c.save({"conan.lock": lock})
        c.run(f"lock remove {args}")
        lock = c.load("conan.lock")
        for remove in removed:
            assert remove not in lock
        rest = {"math/1.0@user", "math/2.0@team/stable",
                "other/1.0@user", "pkg/1.0@team", "engine/1.0"}.difference(removed)
        for pkg in rest:
            assert pkg in lock


class TestLockUpdate:
    @pytest.mark.parametrize("kind, old, new", [
        ("requires", "math/1.0", "math/1.1"),
        ("build-requires", "cmake/1.0", "cmake/1.1"),
        ("python-requires", "mytool/1.0", "mytool/1.1"),
    ])
    def test_lock_update(self, kind, old, new):
        c = TestClient(light=True)
        lock = textwrap.dedent("""\
            {
                "version": "0.5",
                "requires": [
                    "math/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "math/1.0#12345%1702683584.3411012",
                    "engine/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ],
                "build_requires": [
                    "cmake/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "ninja/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ],
                "python_requires": [
                    "mytool/1.0#85d927a4a067a531b1a9c7619522c015%1702683583.3411012",
                    "othertool/1.0#fd2b006646a54397c16a1478ac4111ac%1702683583.3544693"
                ]
            }
            """)
        c.save({"conan.lock": lock})
        c.run(f"lock update --{kind}={new}")
        lock = c.load("conan.lock")
        assert old not in lock
        assert new in lock



class TestLockUpgrade:
    @pytest.mark.parametrize("kind, pkg, old, new", [
        ("requires", "math", "math/1.0", "math/1.1"),
        ("build-requires", "cmake", "cmake/1.0", "cmake/1.1"),     # TODO there is not a --build-requires
        # ("python-requires", "mytool", "mytool/1.0", "mytool/1.1"), # TODO nor a --python-requires
    ])
    def test_lock_upgrade(self, kind, pkg, old, new):
        c = TestClient(light=True)
        c.save({f"{pkg}/conanfile.py": GenConanfile(pkg)})

        c.run(f"export {pkg} --version=1.0")
        rev0 = c.exported_recipe_revision()
        kind_create = "tool-requires" if "build-requires" == kind else kind
        c.run(f"lock create --{kind_create}={pkg}/[*]")
        lock = c.load("conan.lock")
        assert f"{old}#{rev0}" in lock

        c.run(f"export {pkg} --version=1.1")
        rev1 = c.exported_recipe_revision()
        c.run(f"lock upgrade --{kind_create}={pkg}/[*] --update-{kind}={pkg}/[*]")
        lock = c.load("conan.lock")
        print(lock)
        assert f"{old}#{rev0}" not in lock
        assert f"{new}#{rev1}" in lock


    def test_lock_upgrade_path(self):
        c = TestClient(light=True)
        c.save({"liba/conanfile.py": GenConanfile("liba"),
                "libb/conanfile.py": GenConanfile("libb"),
                "libc/conanfile.py": GenConanfile("libc"),
                "libd/conanfile.py": GenConanfile("libd")})
        c.run(f"export liba --version=1.0")
        c.run(f"export libb --version=1.0")
        c.run(f"export libc --version=1.0")
        c.run(f"export libd --version=1.0")
        c.save(
            {
                f"conanfile.py": GenConanfile()
                .with_requires(f"liba/[>=1.0 <2]")
                .with_requires("libb/[<1.2]")
                .with_tool_requires("libc/[>=1.0]")
                .with_python_requires("libd/[>=1.0 <1.2]")
            }
        )

        c.run("lock create .")
        lock = c.load("conan.lock")
        assert "liba/1.0" in lock
        assert "libb/1.0" in lock
        assert "libc/1.0" in lock
        assert "libd/1.0" in lock

        # Check versions are updated accordingly
        c.run(f"export liba --version=1.9")
        c.run(f"export libb --version=1.1")
        c.run(f"export libb --version=1.2")
        c.run(f"export libc --version=1.1")
        c.run(f"export libd --version=1.1")
        c.run("lock upgrade . --update-requires=liba/1.0 --update-requires=libb/[*] --update-build-requires=libc/[*] --update-python-requires=libd/1.0")
        lock = c.load("conan.lock")
        assert "liba/1.9" in lock
        assert "libb/1.1" in lock
        assert "libc/1.1" in lock
        assert "libd/1.1" in lock

        # Check version conanfile version range is respected
        c.run(f"export libd --version=1.2")
        c.run("lock upgrade . --update-python-requires=libd/*")
        lock = c.load("conan.lock")
        assert "libd/1.1" in lock
        assert "libd/1.2" not in lock


    def test_lock_upgrade_new_requirement(self):
        c = TestClient(light=True)
        c.save({"liba/conanfile.py": GenConanfile("liba").with_requires("libb/1.0"),
                "libb/conanfile.py": GenConanfile("libb").with_requires("libc/1.0"),
                "libc/conanfile.py": GenConanfile("libc"),
                "libd/conanfile.py": GenConanfile("libd")})
        c.run(f"export liba --version=1.0")
        c.run(f"export libb --version=1.0")
        c.run(f"export libc --version=1.0")
        c.run(f"export libd --version=1.0")
        c.save({f"conanfile.py": GenConanfile().with_requires(f"liba/[>=1.0 <2]")})
        c.run("lock create .")

        c.save({"libb/conanfile.py": GenConanfile("libb").with_requires("libd/1.0")})
        c.run(f"export libb --version=2.0")
        c.run("lock upgrade --requires='libb/[>=2]' --update-requires='libb/*'")
        lock = c.load("conan.lock")
        assert "libb/2.0" in lock
        assert "libd/1.0" in lock
        assert "libc/1.0" in lock  # TODO: libc should be removed from lockfile? It is not required anymore...

    def test_config_upgrade(self):
        """ Test that it is possible to lock also config-requires
        """
        c = TestClient(light=True)

        c.save({"config/conanfile.py": GenConanfile("config").with_package_type("configuration"),
                "conanfile.py": GenConanfile("pkg", "1.0")})
        c.run("create config --version=1.0")
        c.run("create config --version=2.0")
        c.run("config install-pkg config/1.0 --lockfile-out=conan.lock")

        def _check(refs):
            reqs = json.loads(c.load("conan.lock"))["config_requires"]
            for a, b in zip(refs, reqs):
                assert a in b

        def _check_cache(refs):
            reqs = json.loads(c.load_home("config_version.json"))["config_version"]
            for a, b in zip(refs, reqs):
                assert a in b

        _check(["config/1.0"])
        _check_cache(["config/1.0"])

        c.run("lock upgrade-config --requires=config/[*] --update-config-requires=config/*")
        _check(["config/2.0"])
        _check_cache(["config/1.0"])

        c.run("config install-pkg config/1.0 --lockfile=conan.lock", assert_error=True)
        assert "ERROR: Requirement 'config/1.0' not in lockfile 'config_requires'" in c.out
        c.run("config install-pkg config/2.0 --lockfile=conan.lock")
        _check(["config/2.0"])
        _check_cache(["config/2.0"])

        # Force downgrade
        c.run("config install-pkg config/1.0 --lockfile= --lockfile-out=conan.lock")
        _check(["config/1.0"])
        _check_cache(["config/1.0"])

        c.save({"conanconfig.yml": "packages:\n    - config/[*]"})
        c.run("lock upgrade-config . --update-config-requires=config/1.0")
        _check(["config/2.0"])
        _check_cache(["config/1.0"])
        c.run("config install-pkg . --lockfile=conan.lock")
        _check_cache(["config/2.0"])



import os

import pytest

from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load


@pytest.fixture(scope="module")
def setup():
    client = TestClient(default_server_user=True)
    conanfile = GenConanfile().with_settings("os", "arch").with_package_file("hellohello0.h", "x")
    client.save({"conanfile.py": conanfile})
    ref = RecipeReference.loads("hello0/0.1@lasote/stable")
    client.run("export . --name=hello0 --version=0.1 --user=lasote --channel=stable")
    client.run("install --requires={} -s os=Windows --build missing".format(ref))
    client.run("install --requires={} -s os=Linux --build missing".format(ref))
    client.run("install --requires={} -s os=Linux -s arch=x86 --build missing".format(ref))
    client.run("upload {} -r default".format(ref))
    latest_rrev = client.cache.get_latest_recipe_revision(ref)
    packages = client.cache.get_package_references(latest_rrev)
    package_ids = [package.package_id for package in packages]
    return client, ref, package_ids, str(conanfile)


@pytest.mark.artifactory_ready
def test_download_recipe_twice(setup):
    client, ref, package_ids, conanfile = setup
    new_client = TestClient(servers=client.servers, inputs=["admin", "password"])
    new_client.run("download hello0/0.1@lasote/stable -r default")
    ref = RecipeReference.loads("hello0/0.1@lasote/stable")

    conanfile_path = new_client.get_latest_ref_layout(ref).conanfile()
    assert conanfile == load(conanfile_path)

    new_client.run("download hello0/0.1@lasote/stable -r default")
    assert conanfile == load(conanfile_path)

    new_client.run("download hello0/0.1@lasote/stable -r default")
    assert conanfile == load(conanfile_path)


@pytest.mark.artifactory_ready
def test_download_packages_twice(setup):
    client, ref, package_ids, _ = setup
    new_client = TestClient(servers=client.servers, inputs=["admin", "password"])
    expected_header_contents = "x"

    new_client.run("download hello0/0.1@lasote/stable:* -r default")
    pref = client.get_latest_package_reference("hello0/0.1@lasote/stable", package_id=package_ids[0])
    package_folder = new_client.get_latest_pkg_layout(pref).package()
    got_header = load(os.path.join(package_folder, "hellohello0.h"))
    assert expected_header_contents == got_header

    new_client.run("download hello0/0.1@lasote/stable:* -r default")
    got_header = load(os.path.join(package_folder, "hellohello0.h"))
    assert expected_header_contents == got_header

    new_client.run("download hello0/0.1@lasote/stable:* -r default")
    got_header = load(os.path.join(package_folder, "hellohello0.h"))
    assert expected_header_contents == got_header



import os
import platform
import textwrap
import pytest

from conan.test.utils.tools import TestClient
from conan.internal.util.files import load


def exe_suffix():
    return '.exe' if platform.system() == 'Windows' else ''


COMPILER_MAP = {
    'gcc': ('gcc', 'g++'),
    'clang': ('clang', 'clang++')
}

compiler_path = "C:/opt/bin" if platform.system() == 'Windows' else "/opt/bin"


@pytest.mark.parametrize('compiler, version, system', [
    ('gcc', '13', 'Linux'),
    ('clang', '15', 'Linux'),
])
def test_toolchain_from_path(compiler, version, system):
    client = TestClient()
    path = client.current_folder

    profile = textwrap.dedent(f'''
    [settings]
    compiler={compiler}
    compiler.version={version}
    compiler.libcxx=libstdc++
    os={system}
    [buildenv]
    PATH={path}
    ''')

    cc, cxx = COMPILER_MAP[compiler]

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "compiler", "os"
        name = "mylib"
        version = "0.1"
    ''')

    client.save({cc + exe_suffix(): ''})
    client.save({cxx + exe_suffix(): ''})
    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile -g QbsProfile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)

    settings_content = load(settings_path)
    assert f'qbs.toolchainType:{compiler}' in settings_content
    toolchain_path = client.current_folder.replace('\\', '/')
    assert f'cpp.toolchainInstallPath:{toolchain_path}' in settings_content
    assert f'cpp.compilerName:{cxx}' in settings_content
    assert f'cpp.cCompilerName:{cc}' in settings_content
    assert f'cpp.cxxCompilerName:{cxx}' in settings_content


@pytest.mark.parametrize('compiler, version, system, cc, cxx', [
    ('gcc', '13', 'Linux', 'gcc', 'g++'),
    ('clang', '15', 'Linux', 'clang', 'clang++'),
])
def test_toolchain_from_conf(compiler, version, system, cc, cxx):
    profile = textwrap.dedent(f'''
    [settings]
    compiler={compiler}
    compiler.version={version}
    compiler.libcxx=libstdc++
    os={system}
    [conf]
    tools.build:compiler_executables ={{"c": "{compiler_path}/{cc}", "cpp": "{compiler_path}/{cxx}"}}
    ''')

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "compiler", "os"
        name = "mylib"
        version = "0.1"
    ''')

    client = TestClient()
    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile -g QbsProfile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)

    settings_content = load(settings_path)
    assert f'qbs.toolchainType:{compiler}' in settings_content
    assert f'cpp.toolchainInstallPath:{compiler_path}' in settings_content
    assert f'cpp.compilerName:{cxx}' in settings_content
    assert f'cpp.cCompilerName:{cc}' in settings_content
    assert f'cpp.cxxCompilerName:{cxx}' in settings_content


@pytest.mark.parametrize('compiler, version, system, cc, cxx', [
    ('gcc', '13', 'Linux', 'gcc', 'g++'),
    ('clang', '15', 'Linux', 'clang', 'clang++'),
])
def test_toolchain_from_env(compiler, version, system, cc, cxx):
    profile = textwrap.dedent(f'''
    [settings]
    compiler={compiler}
    compiler.version={version}
    compiler.libcxx=libstdc++
    os={system}
    [buildenv]
    CC={compiler_path}/{cc}
    CXX={compiler_path}/{cxx}
    ''')

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "compiler", "os"
        name = "mylib"
        version = "0.1"
    ''')

    client = TestClient()
    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile -g QbsProfile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)

    settings_content = load(settings_path)
    assert f'qbs.toolchainType:{compiler}' in settings_content
    assert f'cpp.toolchainInstallPath:{compiler_path}' in settings_content
    assert f'cpp.compilerName:{cxx}' in settings_content
    assert f'cpp.cCompilerName:{cc}' in settings_content
    assert f'cpp.cxxCompilerName:{cxx}' in settings_content


@pytest.mark.parametrize('system, compiler, version, build_type, arch, cppstd', [
    ('Linux', 'gcc', '13', 'Release', 'x86_64', '17'),
    ('Linux', 'gcc', '13', 'Debug', 'x86_64', '14'),
    ('Linux', 'gcc', '13', 'Debug', 'x86', '20'),
    ('Linux', 'gcc', '13', 'Release', 'avr', '11'),
])
def test_options_from_settings(system, compiler, version, build_type, arch, cppstd):
    client = TestClient()

    cc, cxx = COMPILER_MAP[compiler]

    profile = textwrap.dedent(f'''
    [settings]
    arch={arch}
    build_type={build_type}
    compiler={compiler}
    compiler.version={version}
    compiler.libcxx=libstdc++
    compiler.cppstd={cppstd}
    os={system}
    [conf]
    tools.build:compiler_executables ={{"c": "{compiler_path}/{cc}", "cpp": "{compiler_path}/{cxx}"}}
    ''')

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        name = "mylib"
        version = "0.1"
    ''')

    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile -g QbsProfile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)
    settings_content = load(settings_path)

    assert f'qbs.architecture:{arch}' in settings_content
    build_variant = build_type.lower()
    assert f'qbs.buildVariant:{build_variant}' in settings_content
    target_platform = system.lower()
    assert f'qbs.targetPlatform:{target_platform}' in settings_content
    assert 'cpp.cxxLanguageVersion:c++' + cppstd in settings_content
    # TODO: cpp.runtimeLibrary (MSVC only)


def test_options_from_conf():
    client = TestClient()

    profile = textwrap.dedent(f'''
    [settings]
    arch=x86_64
    build_type=Release
    compiler=gcc
    compiler.version=13
    compiler.libcxx=libstdc++
    compiler.cppstd=17
    os=Linux
    [conf]
    tools.build:compiler_executables ={{"c": "{compiler_path}/gcc", "cpp": "{compiler_path}/g++"}}
    tools.build:cflags=['-Dfoo', '-Dbar']
    tools.build:cxxflags=['-Dfoo', '-Dbaz']
    tools.build:sharedlinkflags=['-s']
    tools.build:exelinkflags=['-Wl,-s']
    ''')

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        name = "mylib"
        version = "0.1"
    ''')

    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile -g QbsProfile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)
    settings_content = load(settings_path)

    assert "cpp.cFlags:['-Dfoo', '-Dbar']" in settings_content
    assert "cpp.cxxFlags:['-Dfoo', '-Dbaz']" in settings_content
    assert "cpp.linkerFlags:['-s']" in settings_content
    assert "cpp.driverLinkerFlags:['-s']" in settings_content


def test_options_extra():
    client = TestClient()

    profile = textwrap.dedent(f'''
    [settings]
    arch=x86_64
    build_type=Release
    compiler=gcc
    compiler.version=13
    compiler.libcxx=libstdc++
    compiler.cppstd=17
    os=Linux
    [conf]
    tools.build:compiler_executables ={{"c": "{compiler_path}/gcc", "cpp": "{compiler_path}/g++"}}
    ''')

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    from conan.tools.qbs import QbsProfile
    class Recipe(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        name = "mylib"
        version = "0.1"

        def generate(self):
            profile = QbsProfile(self)
            profile.extra_cflags=['-Dfoo', '-Dbar']
            profile.extra_cxxflags=['-Dfoo', '-Dbaz']
            profile.extra_defines=['FOO', 'QUX']
            profile.extra_sharedlinkflags=['-s']
            profile.extra_exelinkflags=['-Wl,-s']
            profile.generate()
    ''')

    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)
    settings_content = load(settings_path)

    assert "cpp.cFlags:['-Dfoo', '-Dbar']" in settings_content
    assert "cpp.cxxFlags:['-Dfoo', '-Dbaz']" in settings_content
    assert "cpp.defines:['FOO', 'QUX']" in settings_content
    assert "cpp.linkerFlags:['-s']" in settings_content
    assert "cpp.driverLinkerFlags:['-s']" in settings_content


def test_sysroot():
    client = TestClient()

    profile = textwrap.dedent(f'''
    [settings]
    arch=x86_64
    build_type=Release
    compiler=gcc
    compiler.version=13
    compiler.libcxx=libstdc++
    compiler.cppstd=17
    os=Linux
    [conf]
    tools.build:compiler_executables ={{"c": "{compiler_path}/gcc", "cpp": "{compiler_path}/g++"}}
    tools.build:sysroot=\\opt\\usr\\local
    ''')

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        name = "mylib"
        version = "0.1"
    ''')

    client.save({'profile': profile})
    client.save({'conanfile.py': conanfile})
    client.run('install . -pr:a=profile -g QbsProfile')

    settings_path = os.path.join(client.current_folder, 'qbs_settings.txt')
    assert os.path.exists(settings_path)
    settings_content = load(settings_path)

    assert "qbs.sysroot:/opt/usr/local" in settings_content



import json
import os
import shutil

import pytest

from conan.api.conan_api import ConanAPI
from conan.internal.errors import NotFoundException
from conan.api.model import PkgReference
from conan.api.model import RecipeReference
from conan.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient, GenConanfile
from conan.test.utils.env import environment_update


class TestRemoveWithoutUserChannel:

    @pytest.mark.parametrize("dry_run", [True, False])
    def test_local(self, dry_run):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . --name=lib --version=1.0")
        ref_layout = client.exported_layout()
        pkg_layout = client.created_layout()
        extra = " --dry-run" if dry_run else ""
        client.run("remove lib/1.0 -c" + extra)
        assert os.path.exists(ref_layout.base_folder) == dry_run
        assert os.path.exists(pkg_layout.base_folder) == dry_run

    @pytest.mark.parametrize("confirm", [True, False])
    def test_local_dryrun_output(self, confirm):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . --name=lib --version=1.0")
        client.run("remove lib/1.0 --dry-run", inputs=["yes" if confirm else "no"])
        assert "(yes/no)" in client.out  # Users are asked even when dry-run is set
        if confirm:
            assert "Removed recipe and all binaries" in client.out  # Output it printed for dry-run
        else:
            assert "Removed recipe and all binaries" not in client.out

    @pytest.mark.artifactory_ready
    def test_remote(self):
        client = TestClient(default_server_user=True)
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . --name=lib --version=1.0")
        client.run("upload lib/1.0 -r default -c")
        client.run("remove lib/1.0 -c")
        # we can still install it
        client.run("install --requires=lib/1.0@")
        assert "lib/1.0: Retrieving package" in client.out
        client.run("remove lib/1.0 -c")

        # Now remove remotely, dry run first
        client.run("remove lib/1.0 -c -r default --dry-run")

        # we can still install it
        client.run("install --requires=lib/1.0@")
        assert "lib/1.0: Retrieving package" in client.out
        client.run("remove lib/1.0 -c")

        # Now remove remotely, for real this time
        client.run("remove lib/1.0 -c -r default")

        client.run("install --requires=lib/1.0@", assert_error=True)
        assert "Unable to find 'lib/1.0' in remotes" in client.out


class TestRemovePackageRevisions:

    @pytest.mark.artifactory_ready
    def test_remove_package_revision(self):
        """ Remove package ID based on recipe revision. The package must be deleted, but
            the recipe must be preserved
        """
        ref = RecipeReference.loads("foobar/0.1@user/testing#4d670581ccb765839f2239cc8dff8fbd")
        pref_arg = PkgReference(ref, NO_SETTINGS_PACKAGE_ID)
        pref = PkgReference(ref, NO_SETTINGS_PACKAGE_ID, "0ba8627bd47edc3a501e8f0eb9a79e5e")
        c = TestClient(light=True, default_server_user=True)
        c.save({"conanfile.py": GenConanfile()})
        c.run("create . --name=foobar --version=0.1 --user=user --channel=testing")
        assert c.package_exists(pref)
        c.run("upload * -r default -c")
        c.run(f"list {ref}:*")
        assert NO_SETTINGS_PACKAGE_ID in c.out

        # remove from cache
        c.run(f"remove -c {pref_arg.repr_notime()}")
        assert not c.package_exists(pref)

        # remove From server
        c.run(f"remove -c {pref_arg.repr_notime()} -r=default")
        c.run(f"list {ref}:*")
        assert NO_SETTINGS_PACKAGE_ID not in c.out

    @pytest.mark.artifactory_ready
    def test_remove_all_packages_but_the_recipe_at_remote(self):
        """ Remove all the packages but not the recipe in a remote
        """
        c = TestClient(default_server_user=True)
        c.save({"conanfile.py": GenConanfile("foobar", "0.1").with_settings("arch")})
        c.run("create . --user=user --channel=testing -s arch=x86_64")
        c.run("create . --user=user --channel=testing -s arch=x86")
        c.run("upload * -r default -c")

        c.run("list *:* -r default")
        assert "foobar/0.1@user/testing" in c.out
        assert "arch: x86_64" in c.out
        assert "arch: x86" in c.out

        c.run("remove -c foobar/0.1@user/testing:* -r default")
        c.run("list *:* -r default")
        assert "foobar/0.1@user/testing" in c.out
        assert "arch: x86_64" not in c.out
        assert "arch: x86" not in c.out


# populated packages of bar
bar_rrev = "bar/1.1#7db54b020cc95b8bdce49cd6aa5623c0"
bar_rrev2 = "bar/1.1#78b42a981b29d2cb00fda10b72f1e72a"
pkg_id_debug = "9e186f6d94c008b544af1569d1a6368d8339efc5"
pkg_id_release = "efa83b160a55b033c4ea706ddb980cd708e3ba1b"
bar_rrev2_debug = '{}:{}'.format(bar_rrev2, pkg_id_debug)
bar_rrev2_release = '{}:{}'.format(bar_rrev2, pkg_id_release)

bar_prev1 = "ee1ca5821b390a75d7168f4567f9ba75"
bar_prev2 = "f523273047249d5a136fe48d33f645bb"
bar_rrev2_release_prev1 = "{}#{}".format(bar_rrev2_release, bar_prev1)
bar_rrev2_release_prev2 = "{}#{}".format(bar_rrev2_release, bar_prev2)


@pytest.fixture(scope="module")
def _populated_client_base():
    """
    foo/1.0@ (one revision) no packages
    foo/1.0@user/channel (one revision)  no packages
    fbar/1.1@ (one revision)  no packages

    bar/1.0@ (two revision) => Debug, Release => (two package revision each)
    """
    # To generate different package revisions
    package_lines = 'save(self, os.path.join(self.package_folder, "foo.txt"), ' \
                    'os.getenv("foo_test", "Na"))'
    client = TestClient(default_server_user=True)
    conanfile = str(GenConanfile().with_settings("build_type")
                                  .with_package(package_lines)
                                  .with_import("from conan.tools.files import save")
                                  .with_import("import os")
                                  .with_import("import time"))
    client.save({"conanfile.py": conanfile})
    client.run("export . --name foo --version 1.0")
    client.run("export . --name foo --version 1.0 --user user --channel channel")
    client.run("export . --name fbar --version 1.1")

    # Two package revisions for bar/1.1 (Release)
    for _i in range(2):
        with environment_update({'foo_test': str(_i)}):
            client.run("create . --name=bar --version=1.1 -s build_type=Release")
    client.run("create . --name=bar --version=1.1 -s build_type=Debug")

    prefs = _get_revisions_packages(client, bar_rrev2_release, False)
    assert set(prefs) == {bar_rrev2_release_prev1, bar_rrev2_release_prev2}

    # Two recipe revisions for bar/1.1
    client.save({"conanfile.py": conanfile + "\n # THIS IS ANOTHER RECIPE REVISION"})
    client.run("create . --name=bar --version=1.1 -s build_type=Debug")

    client.run("upload '*#*:*#*' -c -r default")

    return client


@pytest.fixture()
def populated_client(_populated_client_base):
    """ this is much faster than creating and uploading everythin
    """
    client = TestClient(default_server_user=True)
    shutil.rmtree(client.cache_folder)
    shutil.copytree(_populated_client_base.cache_folder, client.cache_folder)
    shutil.rmtree(client.servers["default"].test_server._base_path)
    shutil.copytree(_populated_client_base.servers["default"].test_server._base_path,
                    client.servers["default"].test_server._base_path)
    client.update_servers()
    return client


@pytest.mark.parametrize("with_remote", [True, False])
@pytest.mark.parametrize("data", [
    {"remove": "foo*", "recipes": ['bar/1.1', 'fbar/1.1']},
    {"remove": "foo/*", "recipes": ['bar/1.1', 'fbar/1.1']},
    {"remove": "*", "recipes": []},
    {"remove": "*/*", "recipes": []},
    {"remove": "*/*#*", "recipes": []},
    {"remove": "*/*#z*", "recipes": ['foo/1.0@user/channel', 'foo/1.0', 'bar/1.1', 'fbar/1.1']},
    {"remove": "f*", "recipes": ["bar/1.1"]},
    {"remove": "*/1.1", "recipes": ["foo/1.0", "foo/1.0@user/channel"]},
    {"remove": "*/*@user/*", "recipes": ["foo/1.0", "fbar/1.1", "bar/1.1"]},
    {"remove": "*/*@*", "recipes": ['foo/1.0', 'fbar/1.1', 'bar/1.1']},
    {"remove": "*/*#*:*", "recipes": ['bar/1.1', 'foo/1.0@user/channel', 'foo/1.0', 'fbar/1.1']},
    {"remove": "foo/1.0@user/channel:*", "recipes": ['bar/1.1', 'foo/1.0@user/channel', 'foo/1.0',
                                                     'fbar/1.1']},
    {"remove": "foo/*@", "recipes": ['foo/1.0@user/channel', 'bar/1.1', 'fbar/1.1']},
    {"remove": "foo", "recipes": ['bar/1.1', 'fbar/1.1']},
    {"remove": "*#", "recipes": []},
    {"remove": "*/*#", "recipes": []},
])
def test_new_remove_recipes_expressions(populated_client, with_remote, data):
    r = "-r default" if with_remote else ""
    assert "error" not in data
    populated_client.run("remove {} -c {}".format(data["remove"], r))
    populated_client.run(f"list {r} --format=json")
    origin = "default" if with_remote else "Local Cache"
    assert set(json.loads(populated_client.stdout)[origin]) == set(data["recipes"])


@pytest.mark.parametrize("with_remote", [True, False])
@pytest.mark.parametrize("data", [
    {"remove": "bar/*#*", "rrevs": []},
    {"remove": "bar/1.1#z*", "rrevs": [bar_rrev, bar_rrev2]},
    {"remove": "bar/1.1#*9*", "rrevs": []},
    {"remove": "bar/1.1#*2a", "rrevs": [bar_rrev]},
    {"remove": "bar*#*50", "rrevs": [bar_rrev, bar_rrev2]},
    {"remove": "bar*#latest", "rrevs": [bar_rrev2]},  # latest is bar_rrev
    {"remove": "bar*#!latest", "rrevs": [bar_rrev]},  # latest is bar_rrev
    {"remove": "bar*#~latest", "rrevs": [bar_rrev]},  # latest is bar_rrev
])
def test_new_remove_recipe_revisions_expressions(populated_client, with_remote, data):
    r = "-r default" if with_remote else ""
    error = data.get("error", False)
    populated_client.run("remove {} -c {}".format(data["remove"], r), assert_error=error)
    if not error:
        rrevs = _get_revisions_recipes(populated_client, "bar/1.1", with_remote)
        assert rrevs == set(data["rrevs"])


@pytest.mark.parametrize("with_remote", [True, False])
@pytest.mark.parametrize("data", [
    {"remove": "bar/1.1#*:*", "prefs": []},
    {"remove": "bar/1.1#*:*#*", "prefs": []},
    {"remove": "bar/1.1#z*:*", "prefs": [bar_rrev2_debug, bar_rrev2_release]},
    {"remove": "bar/1.1#*:*#kk*", "prefs": [bar_rrev2_debug, bar_rrev2_release]},
    {"remove": "bar/1.1#*:{}*".format(pkg_id_release), "prefs": [bar_rrev2_debug]},
    {"remove": "bar/1.1#*:*{}".format(pkg_id_release[-12:]), "prefs": [bar_rrev2_debug]},
    {"remove": "{}:*{}*".format(bar_rrev2, pkg_id_debug[5:15]), "prefs": [bar_rrev2_release]},
    {"remove": "*/*#*:*{}*".format(pkg_id_debug[5:15]), "prefs": [bar_rrev2_release]},
    {"remove": '*/*#*:* -p build_type="fake"', "prefs": [bar_rrev2_release, bar_rrev2_debug]},
    {"remove": '*/*#*:* -p build_type="Release"', "prefs": [bar_rrev2_debug]},
    {"remove": '*/*#*:* -p build_type="Debug"', "prefs": [bar_rrev2_release]},
    # Errors
    {"remove": '*/*#*:*#* -p', "error": True,
     "error_msg": "argument -p/--package-query: expected one argument"},
    {"remove": "bar/1.1#*:", "prefs": []},
    {"remove": "bar/1.1#*:234234*#", "prefs": [bar_rrev2_debug, bar_rrev2_release]},
    {"remove": "bar/1.1:234234*", "prefs": [bar_rrev2_debug, bar_rrev2_release]},
    {"remove": "bar/1.1:* -p os=Windows", "prefs": [bar_rrev2_debug, bar_rrev2_release]},
])
def test_new_remove_package_expressions(populated_client, with_remote, data):
    # Remove the ones we are not testing here
    r = "-r default" if with_remote else ""
    pids = _get_all_packages(populated_client, bar_rrev2, with_remote)
    assert pids == {bar_rrev2_debug, bar_rrev2_release}

    error = data.get("error", False)
    populated_client.run("remove {} -c {}".format(data["remove"], r), assert_error=error)
    if not error:
        pids = _get_all_packages(populated_client, bar_rrev2, with_remote)
        assert pids == set(data["prefs"])
    elif data.get("error_msg"):
        assert data.get("error_msg") in populated_client.out


@pytest.mark.parametrize("with_remote", [True, False])
@pytest.mark.parametrize("data", [
    {"remove": '{}#*kk*'.format(bar_rrev2_release), "prevs": [bar_rrev2_release_prev1,
                                                              bar_rrev2_release_prev2]},
    {"remove": '{}#*'.format(bar_rrev2_release), "prevs": []},
    {"remove": '{}*#{}* -p "build_type=Debug"'.format(bar_rrev2_release, bar_prev2[:3]),
     "prevs": [bar_rrev2_release_prev1, bar_rrev2_release_prev2]},
    {"remove": '{}*#{}* -p "build_type=Release"'.format(bar_rrev2_release, bar_prev2[:3]),
     "prevs": [bar_rrev2_release_prev1]},
    {"remove": '{}*#* -p "build_type=Release"'.format(bar_rrev2_release), "prevs": []},
    {"remove": '{}*#* -p "build_type=Debug"'.format(bar_rrev2_release),
     "prevs": [bar_rrev2_release_prev1, bar_rrev2_release_prev2]},
    # Errors
    {"remove": '{}#'.format(bar_rrev2_release), "prevs": []},
    {"remove": '{}#latest'.format(bar_rrev2_release), "prevs": [bar_rrev2_release_prev1]},
    {"remove": '{}#!latest'.format(bar_rrev2_release), "prevs": [bar_rrev2_release_prev2]},
    {"remove": '{}#~latest'.format(bar_rrev2_release), "prevs": [bar_rrev2_release_prev2]},
])
def test_new_remove_package_revisions_expressions(populated_client, with_remote, data):
    # Remove the ones we are not testing here
    r = "-r default" if with_remote else ""
    populated_client.run("remove f/* -c {}".format(r))

    prefs = _get_revisions_packages(populated_client, bar_rrev2_release, with_remote)
    assert set(prefs) == {bar_rrev2_release_prev1, bar_rrev2_release_prev2}

    assert "error" not in data
    populated_client.run("remove {} -c {}".format(data["remove"], r))
    prefs = _get_revisions_packages(populated_client, bar_rrev2_release, with_remote)
    assert set(prefs) == set(data["prevs"])


def test_package_query_no_package_ref(populated_client):
    populated_client.run("remove * -p 'compiler=clang'", assert_error=True)
    assert "--package-query supplied but the pattern does not match packages" in populated_client.out


def _get_all_packages(client, ref, with_remote):
    ref = RecipeReference.loads(ref)
    with client.mocked_servers():
        api = ConanAPI(client.cache_folder)
        remote = api.remotes.get("default") if with_remote else None
        try:
            return set([r.repr_notime() for r in api.list._packages_configurations(ref, remote=remote)])  # noqa
        except NotFoundException:
            return set()


def _get_revisions_recipes(client, ref, with_remote):
    ref = RecipeReference.loads(ref)
    with client.mocked_servers():
        api = ConanAPI(client.cache_folder)
        remote = api.remotes.get("default") if with_remote else None
        try:
            return set([r.repr_notime() for r in api.list.recipe_revisions(ref, remote=remote)])
        except NotFoundException:
            return set()


def _get_revisions_packages(client, pref, with_remote):
    pref = PkgReference.loads(pref)
    with client.mocked_servers():
        api = ConanAPI(client.cache_folder)
        remote = api.remotes.get("default") if with_remote else None
        try:
            return set([r.repr_notime() for r in api.list.package_revisions(pref, remote=remote)])
        except NotFoundException:
            return set()



import os
import pytest
from collections import namedtuple, Counter

from requests.exceptions import HTTPError

from conan.internal.rest.file_uploader import FileUploader
from conan.internal.errors import AuthenticationException, ForbiddenException
from conan.test.utils.mocks import RedirectedTestOutput
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import redirect_output
from conan.internal.util.files import save


class _ResponseMock:
    def __init__(self, status_code, content):
        self.status_code = status_code
        self.content = content

    def raise_for_status(self):
        """Raises stored :class:`HTTPError`, if one occurred."""
        http_error_msg = ''
        if 500 <= self.status_code < 600:
            http_error_msg = u'%s Server Error: %s' % (self.status_code, self.content)

        if http_error_msg:
            raise HTTPError(http_error_msg, response=self)


class _RequesterMock:
    def __init__(self, status_code, content):
        self.response = _ResponseMock(status_code, content)
        self.retry = 0
        self.retry_wait = 0

    def put(self, *args, **kwargs):
        return self.response


class _ConfigMock:

    def get(self, conf_name, default=None, check_type=None):
        return 0


class TestRetryDownload:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.filename = os.path.join(temp_folder(), "anyfile")
        save(self.filename, "anything")

    def test_error_401(self):
        output = RedirectedTestOutput()
        with redirect_output(output):
            uploader = FileUploader(requester=_RequesterMock(401, "content"),
                                    verify=False, config=_ConfigMock())
            with pytest.raises(AuthenticationException, match="content"):
                uploader.upload(url="fake", abs_path=self.filename, retry=2)
            output_lines = output.getvalue().splitlines()
            counter = Counter(output_lines)
            assert counter["ERROR: content"] == 0
            assert counter["Waiting 0 seconds to retry..."] == 0

    def test_error_403_forbidden(self):
        output = RedirectedTestOutput()
        with redirect_output(output):
            uploader = FileUploader(requester=_RequesterMock(403, "content"),
                                    verify=False, config=_ConfigMock())
            with pytest.raises(ForbiddenException, match="content"):
                auth = namedtuple("auth", "bearer")
                uploader.upload(url="fake", abs_path=self.filename, retry=2, auth=auth("token"))
            output_lines = output.getvalue().splitlines()
            counter = Counter(output_lines)
            assert counter["ERROR: content"] == 0
            assert counter["Waiting 0 seconds to retry..."] == 0

    def test_error_403_authentication(self):
        output = RedirectedTestOutput()
        with redirect_output(output):
            uploader = FileUploader(requester=_RequesterMock(403, "content"),
                                    verify=False, config=_ConfigMock())
            with pytest.raises(AuthenticationException, match="content"):
                auth = namedtuple("auth", "bearer")
                uploader.upload(url="fake", abs_path=self.filename, retry=2, auth=auth(None))
            output_lines = output.getvalue().splitlines()
            counter = Counter(output_lines)
            assert counter["ERROR: content"] == 0
            assert counter["Waiting 0 seconds to retry..."] == 0

    def test_error_requests(self):
        class _RequesterMock:

            def put(self, *args, **kwargs):
                raise Exception("any exception")

        output = RedirectedTestOutput()
        with redirect_output(output):
            uploader = FileUploader(requester=_RequesterMock(), verify=False, config=_ConfigMock())
            with pytest.raises(Exception, match="any exception"):
                uploader.upload(url="fake", abs_path=self.filename, retry=2)
            output_lines = output.getvalue().splitlines()
            counter = Counter(output_lines)
            assert counter["WARN: network: any exception"] == 2
            assert counter["Waiting 0 seconds to retry..."] == 2

    def test_error_500(self):
        output = RedirectedTestOutput()
        with redirect_output(output):
            uploader = FileUploader(requester=_RequesterMock(500, "content"), verify=False,
                                    config=_ConfigMock())
            with pytest.raises(Exception, match="500 Server Error: content"):
                uploader.upload(url="fake", abs_path=self.filename, retry=2)
            output_lines = output.getvalue().splitlines()
            counter = Counter(output_lines)
            assert counter["WARN: network: 500 Server Error: content"] == 2
            assert counter["Waiting 0 seconds to retry..."] == 2



import os
import pytest
from datetime import timedelta

from conan.errors import ConanException
from conans.server.conf import ConanServerConfigParser
from conan.test.utils.test_files import temp_folder
from conan.internal.util.config_parser import TextINIParse
from conan.internal.util.files import save

fileconfig = '''
[server]
jwt_secret: mysecret
jwt_expire_minutes: 121
disk_storage_path: %s
ssl_enabled: true
port: 9220
host_name: localhost
public_port: 12345


[write_permissions]
openssl/2.0.1@lasote/testing: pepe

[read_permissions]
*/*@*/*: *
openssl/2.0.1@lasote/testing: pepe

[users]
lasote: defaultpass
pepe: pepepass
'''


class TestServerConf:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.file_path = temp_folder()
        server_conf = os.path.join(self.file_path, '.conan_server/server.conf')
        self.storage_path = os.path.join(self.file_path, "storage")
        save(server_conf, fileconfig % self.storage_path)
        self.environ = {}

    def test_unexpected_section(self):
        text = """
[one]
text=value
[two]
other=var
[three]
var
[moon]
var=walker
"""

        with pytest.raises(ConanException):
            TextINIParse(text, ["one", "two", "three"])
        conf = TextINIParse(text)
        assert conf.one == "text=value"
        assert conf.two == "other=var"
        assert conf.three == "var"
        assert conf.moon == "var=walker"

        # IF an old config file is readed but the section is in the list, just return it empty
        text = """
[one]
text=value
        """
        conf = TextINIParse(text)
        assert conf.two == ""

    def test_values(self):
        config = ConanServerConfigParser(self.file_path, environment=self.environ)
        assert config.jwt_secret == "mysecret"
        assert config.jwt_expire_time == timedelta(minutes=121)
        assert config.disk_storage_path == self.storage_path
        assert config.ssl_enabled
        assert config.port == 9220
        assert config.write_permissions == [("openssl/2.0.1@lasote/testing", "pepe")]
        assert config.read_permissions == [("*/*@*/*", "*"),
                                                    ("openssl/2.0.1@lasote/testing", "pepe")]
        assert config.users == {"lasote": "defaultpass", "pepe": "pepepass"}
        assert config.host_name == "localhost"
        assert config.public_port == 12345
        assert config.public_url == "https://localhost:12345/v2"

        # Now check with environments
        tmp_storage = temp_folder()
        self.environ["CONAN_STORAGE_PATH"] = tmp_storage
        self.environ["CONAN_JWT_SECRET"] = "newkey"
        self.environ["CONAN_JWT_EXPIRE_MINUTES"] = "123"
        self.environ["CONAN_SSL_ENABLED"] = "False"
        self.environ["CONAN_SERVER_PORT"] = "1233"
        self.environ["CONAN_SERVER_USERS"] = "lasote:lasotepass,pepe2:pepepass2"
        self.environ["CONAN_HOST_NAME"] = "remotehost"
        self.environ["CONAN_SERVER_PUBLIC_PORT"] = "33333"

        config = ConanServerConfigParser(self.file_path, environment=self.environ)
        assert config.jwt_secret ==  "newkey"
        assert config.jwt_expire_time == timedelta(minutes=123)
        assert config.disk_storage_path == tmp_storage
        assert not config.ssl_enabled
        assert config.port == 1233
        assert config.write_permissions == [("openssl/2.0.1@lasote/testing", "pepe")]
        assert config.read_permissions == [("*/*@*/*", "*"),
                                                    ("openssl/2.0.1@lasote/testing", "pepe")]
        assert config.users == {"lasote": "lasotepass", "pepe2": "pepepass2"}
        assert config.host_name == "remotehost"
        assert config.public_port == 33333
        assert config.public_url == "http://remotehost:33333/v2"



import textwrap

from conan.test.utils.tools import TestClient


def test_premake_args_without_toolchain():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.premake import Premake

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def run(self, cmd, *args, **kwargs):
                self.output.info(f"Running {cmd}!!")
            def build(self):
                premake = Premake(self)
                premake.luafile = "myproject.lua"
                premake.arguments = {"myarg": "myvalue"}
                premake.configure()
                """)
    c.save({"conanfile.py": conanfile})
    c.run(
        "build . -s compiler=msvc -s compiler.version=193 -s compiler.runtime=dynamic"
    )
    assert (
        'conanfile.py: Running premake5 --file="myproject.lua" vs2022 --myarg=myvalue!!'
        in c.out
    )


def test_premake_build_without_toolchain():
    tc = TestClient()
    conanfile = textwrap.dedent(
        """
        from conan import ConanFile
        from conan.tools.premake import Premake, PremakeToolchain

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def run(self, cmd, *args, **kwargs):
                self.output.info(f"Running {cmd}!!")
            def build(self):
                premake = Premake(self)
                premake.configure()
                premake.build(workspace="App")
                """
    )
    tc.save({"conanfile.py": conanfile})
    tc.run(
        "build . -s compiler=msvc -s compiler.version=193 -s compiler.runtime=dynamic",
        assert_error=True,
    )
    assert "Premake.build() method requires PremakeToolchain to work properly" in tc.out


def test_premake_build():
    tc = TestClient()
    conanfile = textwrap.dedent(
        """
        from conan import ConanFile
        from conan.tools.premake import Premake, PremakeToolchain

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "PremakeToolchain"
            def run(self, cmd, *args, **kwargs):
                self.output.info(f"Running {cmd}!!")
            def build(self):
                premake = Premake(self)
                premake.configure()
                premake.build(workspace="App")
                """
    )
    tc.save({"conanfile.py": conanfile})
    tc.run(
        "build . -s compiler=gcc -s compiler.version=13 -s compiler.libcxx=libstdc++ -s arch=x86_64 -c tools.build:jobs=4",
    )
    assert 'conanfile.premake5.lua" gmake --arch=x86_64!!' in tc.out
    assert "Running make config=release all -j4!!" in tc.out


def test_premake_build_with_targets():
    tc = TestClient()
    conanfile = textwrap.dedent(
        """
        from conan import ConanFile
        from conan.tools.premake import Premake, PremakeToolchain

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "PremakeToolchain"
            def run(self, cmd, *args, **kwargs):
                self.output.info(f"Running {cmd}!!")
            def build(self):
                premake = Premake(self)
                premake.configure()
                premake.build(workspace="App", targets=["app", "test"])
                """
    )
    tc.save({"conanfile.py": conanfile})
    tc.run(
        "build . -s compiler=gcc -s compiler.version=13 -s compiler.libcxx=libstdc++ -s arch=armv8 -c tools.build:jobs=42",
    )
    assert 'conanfile.premake5.lua" gmake --arch=arm64!!' in tc.out
    assert "Running make config=release app test -j42!!" in tc.out


def test_premake_msbuild_platform():
    tc = TestClient()
    windows_profile = textwrap.dedent("""
        [settings]
        arch=x86_64
        build_type=Release
        compiler=msvc
        compiler.cppstd=14
        compiler.runtime=dynamic
        compiler.version=194
        os=Windows
    """)

    conanfile = textwrap.dedent(
        """
        from conan import ConanFile
        from conan.tools.premake import Premake, PremakeToolchain

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "PremakeToolchain"
            def run(self, cmd, *args, **kwargs):
                self.output.info(f"Running {cmd}!!")
            def build(self):
                premake = Premake(self)
                premake.configure()
                premake.build(workspace="App", msbuild_platform="Win64")
                """
    )
    tc.save({"conanfile.py": conanfile, "win": windows_profile})
    tc.run("build . -pr win")
    assert 'conanfile.premake5.lua" vs2022 --arch=x86_64!!' in tc.out
    assert 'Running msbuild.exe "App.sln" -p:Configuration="Release" -p:Platform="Win64"!!' in tc.out


def test_premake_build_with_custom_configuration():
    tc = TestClient()
    conanfile = textwrap.dedent(
        """
        from conan import ConanFile
        from conan.tools.premake import Premake, PremakeDeps, PremakeToolchain

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "PremakeToolchain"
            def run(self, cmd, *args, **kwargs):
                self.output.info(f"Running {cmd}!!")
            def generate(self):
                deps = PremakeDeps(self)
                deps.configuration = "ReleaseDLL"
            def build(self):
                premake = Premake(self)
                premake.configure()
                premake.build(workspace="App", configuration="ReleaseDLL")
                """
    )
    tc.save({"conanfile.py": conanfile})
    tc.run(
        "build . -s compiler=gcc -s compiler.version=13 -s compiler.libcxx=libstdc++ -s arch=armv8 -c tools.build:jobs=4",
    )
    assert 'conanfile.premake5.lua" gmake --arch=arm64!!' in tc.out
    assert "Running make config=releasedll all -j4!!" in tc.out




import os
import textwrap

from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save_files, load


class TestExportsMethod:

    def test_export_method(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import copy

            class MethodConan(ConanFile):
                exports = "file.txt"
                def export(self):
                    copy(self, "LICENSE.md", self.recipe_folder, self.export_folder)
            """)
        client.save({"conanfile.py": conanfile, "LICENSE.md": "license", "file.txt": "file"})
        client.run("export . --name=pkg --version=0.1")
        assert "pkg/0.1: Copied 1 '.txt' file: file.txt" in client.out
        assert "pkg/0.1: Calling export()" in client.out
        assert "Copied 1 '.md' file: LICENSE.md" in client.out

        layout = client.exported_layout()

        exported_files = os.listdir(layout.export())
        assert "file.txt" in exported_files
        assert "LICENSE.md" in exported_files

    def test_export_method_parent_folder(self):
        folder = temp_folder()
        save_files(folder, {"subdir/subdir2/file2.txt": "", "subdir/file1.txt": ""})
        client = TestClient(current_folder=os.path.join(folder, "recipe"))
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import copy

            class MethodConan(ConanFile):
                def export(self):
                    self.output.info("Executing export() method")
                    copy(self, "*.txt", os.path.join(self.recipe_folder, "../subdir"),
                         self.export_folder)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . --name=pkg --version=0.1")
        assert "Copied 2 '.txt' files: file1.txt, file2.txt" in client.out

        layout = client.exported_layout()

        assert os.path.isfile(os.path.join(layout.export(), "file1.txt"))
        assert os.path.isfile(os.path.join(layout.export(), "subdir2", "file2.txt"))

    def test_export_no_settings_options_method(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import copy

            class MethodConan(ConanFile):
                settings = "os"
                def export(self):
                    if self.settings.os == "Windows":
                        copy(self, "LICENSE.md", self.recipe_folder, self.export_folder)
            """)
        client.save({"conanfile.py": conanfile, "LICENSE.md": "license"})
        client.run("export . --name=pkg --version=0.1", assert_error=True)
        assert "ERROR: pkg/0.1: Error in export() method, line 8" in client.out

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import copy

            class MethodConan(ConanFile):
                options = {"myopt": ["myval", "other"]}
                default_options = {"myopt": "myval"}
                def export(self):
                    if self.default_options["myopt"] == "myval":
                        copy(self, "LICENSE.md", self.recipe_folder, self.export_folder)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . --name=pkg --version=0.1", assert_error=True)
        assert "ERROR: pkg/0.1: Error in export() method, line 9" in client.out

        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class MethodConan(ConanFile):
                options = {"myopt": ["myval"]}
                default_options = {"myopt": "myval"}
                def export(self):
                    pass
                def export_sources(self):
                    pass
                def build(self):
                    self.output.info("MYOPT: %s" % self.options.myopt)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=pkg --version=0.1")
        assert "pkg/0.1: MYOPT: myval" in client.out

    def test_export_folders(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
           from conan import ConanFile
           from conan.tools.files import save, load
           import os

           class MethodConan(ConanFile):
               def export(self):
                   content = load(self, os.path.join(os.getcwd(), "data.txt"))
                   save(self, os.path.join(self.export_folder, "myfile.txt"), content)
           """)
        client.save({"recipe/conanfile.py": conanfile, "recipe/data.txt": "mycontent"})
        client.run("export recipe --name=pkg --version=0.1")
        layout = client.exported_layout()
        assert "mycontent" == load(os.path.join(layout.export(), "myfile.txt"))
        client.current_folder = os.path.join(client.current_folder, "recipe")
        client.run("export . --name=pkg --version=0.1")
        layout = client.exported_layout()
        assert "mycontent" == load(os.path.join(layout.export(), "myfile.txt"))

    def test_export_attribute_error(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class MethodConan(ConanFile):
                export = "file.txt"
            """)
        client.save({"conanfile.py": conanfile, "file.txt": "file"})
        client.run("export . --name=pkg --version=0.1", assert_error=True)
        assert "ERROR: conanfile 'export' must be a method" in client.out

    def test_exports_method_error(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class MethodConan(ConanFile):
                def exports(self):
                    pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . --name=pkg --version=0.1", assert_error=True)
        assert "ERROR: conanfile 'exports' shouldn't be a method, use 'export()' instead" in client.out


class TestExportsSourcesMethod:

    def test_export_sources_method(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import copy

            class MethodConan(ConanFile):
                exports_sources = "file.txt"
                def export_sources(self):
                    copy(self, "LICENSE.md", self.recipe_folder, self.export_sources_folder)
            """)
        client.save({"conanfile.py": conanfile, "LICENSE.md": "license", "file.txt": "file"})
        client.run("export . --name=pkg --version=0.1")
        assert "pkg/0.1: Copied 1 '.txt' file: file.txt" in client.out
        assert "Copied 1 '.md' file: LICENSE.md" in client.out

        layout = client.exported_layout()

        exported_files = os.listdir(layout.export_sources())
        assert "file.txt" in exported_files
        assert "LICENSE.md" in exported_files

    def test_export_source_folders(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
           from conan import ConanFile
           from conan.tools.files import save, load
           import os

           class MethodConan(ConanFile):
               def export_sources(self):
                   content = load(self, os.path.join(os.getcwd(), "data.txt"))
                   save(self, os.path.join(self.export_sources_folder, "myfile.txt"), content)
           """)
        client.save({"recipe/conanfile.py": conanfile, "recipe/data.txt": "mycontent"})
        client.run("export recipe --name=pkg --version=0.1")
        layout = client.exported_layout()
        assert "mycontent" == load(os.path.join(layout.export_sources(), "myfile.txt"))
        client.current_folder = os.path.join(client.current_folder, "recipe")
        client.run("export . --name=pkg --version=0.1")
        layout = client.exported_layout()
        assert "mycontent" == load(os.path.join(layout.export_sources(), "myfile.txt"))

    def test_export_sources_attribute_error(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class MethodConan(ConanFile):
                export_sources = "file.txt"
            """)
        client.save({"conanfile.py": conanfile, "file.txt": "file"})
        client.run("export . --name=pkg --version=0.1", assert_error=True)
        assert "ERROR: conanfile 'export_sources' must be a method" in client.out

    def test_exports_sources_method_error(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class MethodConan(ConanFile):
                def exports_sources(self):
                    pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . --name=pkg --version=0.1", assert_error=True)
        assert ("ERROR: conanfile 'exports_sources' shouldn't be a method, "
                "use 'export_sources()' instead") in client.out

    def test_exports_sources_upload_error(self):
        # https://github.com/conan-io/conan/issues/7377
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import load, copy

            class MethodConan(ConanFile):
                def export_sources(self):
                    copy(self, "*", self.recipe_folder, self.export_sources_folder)
                def build(self):
                    self.output.info("CONTENT: %s" % load(self, "myfile.txt"))
            """)
        client.save({"conanfile.py": conanfile,
                     "myfile.txt": "mycontent"})
        client.run("export . --name=pkg --version=0.1")
        assert "Copied 1 '.txt' file: myfile.txt" in client.out
        client.run("upload pkg/0.1 -r default")
        client.run("remove * -c")
        client.run("install --requires=pkg/0.1@ --build='*'")
        assert "pkg/0.1: Sources downloaded from 'default'" in client.out
        assert "pkg/0.1: CONTENT: mycontent" in client.out



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


def test_custom_global_generator():
    c = TestClient()
    generator = textwrap.dedent("""
        class MyCustomGenerator:
            def __init__(self, conanfile):
                self._conanfile = conanfile
            def generate(self):
                pkg = self._conanfile.dependencies["pkg"].ref
                self._conanfile.output.info(f"DEP: {pkg}!!")
        """)
    save(os.path.join(c.paths.custom_generators_path, "mygen.py"), generator)
    conanfile = textwrap.dedent("""
        [requires]
        pkg/0.1

        [generators]
        MyCustomGenerator
        """)
    c.save({"pkg/conanfile.py": GenConanfile("pkg", "0.1"),
            "conanfile.txt": conanfile})
    c.run("create pkg")
    c.run("install .")
    assert "conanfile.txt: Generator 'MyCustomGenerator' calling 'generate()'" in c.out
    assert "conanfile.txt: DEP: pkg/0.1!!" in c.out

    # By CLI also works
    conanfile = textwrap.dedent("""
        [requires]
        pkg/0.1
        """)
    c.save({"conanfile.txt": conanfile})
    c.run("install . -g MyCustomGenerator")
    assert "conanfile.txt: Generator 'MyCustomGenerator' calling 'generate()'" in c.out
    assert "conanfile.txt: DEP: pkg/0.1!!" in c.out

    # In conanfile.py also works
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class MyPkg(ConanFile):
            requires = "pkg/0.1"
            generators = "MyCustomGenerator"
            """)
    c.save({"conanfile.py": conanfile}, clean_first=True)
    c.run("install . ")
    assert "conanfile.py: Generator 'MyCustomGenerator' calling 'generate()'" in c.out
    assert "conanfile.py: DEP: pkg/0.1!!" in c.out


def test_custom_global_generator_imports():
    """
    our custom generator can use python imports
    """
    c = TestClient()
    generator = textwrap.dedent("""
        from _myfunc import mygenerate

        class MyCustomGenerator:
            def __init__(self, conanfile):
                self._conanfile = conanfile
            def generate(self):
                mygenerate(self._conanfile)
        """)
    myaux = textwrap.dedent("""
        def mygenerate(conanfile):
            conanfile.output.info("MYGENERATE WORKS!!")
        """)
    save(os.path.join(c.paths.custom_generators_path, "mygen.py"), generator)
    save(os.path.join(c.paths.custom_generators_path, "_myfunc.py"), myaux)

    c.save({"conanfile.txt": ""})
    c.run("install . -g MyCustomGenerator")
    assert "conanfile.txt: Generator 'MyCustomGenerator' calling 'generate()'" in c.out
    assert "conanfile.txt: MYGENERATE WORKS!!" in c.out


def test_custom_global_generator_multiple():
    """
    multiple custom generators with different names load correctly
    """
    c = TestClient()
    for num in range(3):
        generator = textwrap.dedent(f"""
            class MyGenerator{num}:
                def __init__(self, conanfile):
                    self._conanfile = conanfile
                def generate(self):
                    self._conanfile.output.info(f"MyGenerator{num}!!")
            """)
        save(os.path.join(c.paths.custom_generators_path, f"mygen{num}.py"), generator)
    conanfile = textwrap.dedent("""
        [requires]
        pkg/0.1

        [generators]
        MyGenerator0
        MyGenerator1
        MyGenerator2
        """)
    c.save({"pkg/conanfile.py": GenConanfile("pkg", "0.1"),
            "conanfile.txt": conanfile})
    c.run("create pkg")
    c.run("install .")
    assert "conanfile.txt: Generator 'MyGenerator0' calling 'generate()'" in c.out
    assert "conanfile.txt: Generator 'MyGenerator1' calling 'generate()'" in c.out
    assert "conanfile.txt: Generator 'MyGenerator2' calling 'generate()'" in c.out
    assert "conanfile.txt: MyGenerator0!!" in c.out
    assert "conanfile.txt: MyGenerator1!!" in c.out
    assert "conanfile.txt: MyGenerator2!!" in c.out



from conan.test.utils.tools import TestClient, GenConanfile


class TestRequireOverride:

    def test_override_user_channel(self):
        c = TestClient(light=True)
        c.save({"dep/conanfile.py": GenConanfile(),
                "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_requires("dep1/0.1")
                                                              .with_requires("dep2/0.1@us/chan"),
                "app/conanfile.py": GenConanfile().with_requires("pkg/0.1")
                                                  .with_requirement("dep1/0.1@us/chan",
                                                                    override=True)
                                                  .with_requirement("dep2/0.1", override=True)})
        c.run("create dep --name=dep1 --version=0.1")
        c.run("create dep --name=dep1 --version=0.1 --user=us --channel=chan")
        c.run("create dep --name=dep2 --version=0.1")
        c.run("create dep --name=dep2 --version=0.1 --user=us --channel=chan")
        c.run("export pkg")
        c.run("graph info app")
        c.assert_overrides({"dep1/0.1": ['dep1/0.1@us/chan'],
                            "dep2/0.1@us/chan": ['dep2/0.1']})
        c.assert_listed_require({"dep1/0.1@us/chan": "Cache",
                                 "dep2/0.1": "Cache"})

    def test_can_override_even_versions_with_build_metadata(self):
        # https://github.com/conan-io/conan/issues/5900
        c = TestClient(light=True)
        c.save({"conanfile.py": GenConanfile("lib")})
        c.run("create . --version=1.0+abc")
        c.run("create . --version=1.0+xyz")

        c.save({"conanfile.py": GenConanfile("pkg", "1.0").with_require("lib/1.0+abc")})
        c.run("create .")

        c.save({"conanfile.py": GenConanfile().with_require("pkg/1.0")
                                              .with_requirement("lib/1.0+xyz", override=True)})
        c.run("graph info .")
        c.assert_overrides({"lib/1.0+abc": ['lib/1.0+xyz']})
        c.assert_listed_require({"lib/1.0+xyz": "Cache",
                                 "pkg/1.0": "Cache"})




import platform
import textwrap

import pytest

from conan.internal.api.detect import detect_api
from conan.test.utils.tools import TestClient


class TestProfileDetectAPI:
    @pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
    @pytest.mark.tool("visual_studio", "17")
    def test_profile_detect_compiler(self):

        client = TestClient()
        tpl1 = textwrap.dedent("""
            {% set compiler, version, compiler_exe = detect_api.detect_default_compiler() %}
            {% set runtime, _ = detect_api.default_msvc_runtime(compiler) %}
            [settings]
            compiler={{compiler}}
            compiler.version={{detect_api.default_compiler_version(compiler, version)}}
            compiler.runtime={{runtime}}
            compiler.cppstd={{detect_api.default_cppstd(compiler, version)}}
            # detect_msvc_update returns the real update, like 12 for VS 17.12 so
            # we have to convert to the setting that's 0-10
            compiler.update={{ (detect_api.detect_msvc_update(version) | int) % 10 }}

            [conf]
            tools.microsoft.msbuild:vs_version={{detect_api.default_msvc_ide_version(version)}}
            """)

        client.save({"profile1": tpl1})
        client.run("profile show -pr=profile1 --context=host")
        # FIXME: check update setting
        update = str(int(detect_api.detect_msvc_update("194")) % 10)
        expected = textwrap.dedent(f"""\
            [settings]
            compiler=msvc
            compiler.cppstd=14
            compiler.runtime=dynamic
            compiler.runtime_type=Release
            compiler.update={update}
            compiler.version=194
            [conf]
            tools.microsoft.msbuild:vs_version=17
            """)
        assert expected in client.out

    @pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
    def test_profile_detect_libc(self):
        client = TestClient()
        tpl1 = textwrap.dedent("""
            {% set compiler, version, _ = detect_api.detect_gcc_compiler() %}
            {% set libc, libc_version = detect_api.detect_libc() %}
            [settings]
            os=Linux
            compiler={{compiler}}
            compiler.version={{version}}
            [conf]
            user.confvar:libc={{libc}}
            user.confvar:libc_version={{libc_version}}
            """)

        client.save({"profile1": tpl1})
        client.run("profile show -pr=profile1 --context=host")
        libc_name, libc_version = detect_api.detect_libc()
        assert libc_name is not None
        assert libc_version is not None
        _, version, _ = detect_api.detect_gcc_compiler()
        expected = textwrap.dedent(f"""\
            [settings]
            compiler=gcc
            compiler.version={version}
            os=Linux
            [conf]
            user.confvar:libc={libc_name}
            user.confvar:libc_version={libc_version}
            """)
        assert expected in client.out

    @pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
    def test_profile_detect_darwin_sdk(self):
        client = TestClient()
        tpl1 = textwrap.dedent("""\
            [settings]
            os = "Macos"
            os.sdk_version = {{ detect_api.detect_sdk_version(sdk="macosx")  }}
            """)

        client.save({"profile1": tpl1})
        client.run("profile show -pr=profile1")
        sdk_version = detect_api.detect_sdk_version(sdk="macosx")
        assert f"os.sdk_version={sdk_version}" in client.out


@pytest.mark.parametrize("context", [None, "host", "build"])
@pytest.mark.parametrize("f", ["json", "text"])
def test_profile_show_aggregate_usecase(context, f):
    tc = TestClient(light=True)

    context_arg = f"--context {context}" if context else ""
    tc.run(f'profile show {context_arg} -s:h="os=Windows" -s:b="os=Linux" --format={f}')

    if context == "host":
        if f == "text":
            assert "Host profile:" not in tc.stdout
            assert "Host profile:" in tc.stderr
        assert "Linux" not in tc.out
    if context == "build":
        if f == "text":
            assert "Build profile:" not in tc.stdout
            assert "Build profile:" in tc.stderr
        assert "Windows" not in tc.out

    if context in (None, "host"):
        assert "Windows" in tc.out
    if context in (None, "build"):
        assert "Linux" in tc.out



import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.mark.parametrize("requires", ["requires", "tool_requires"])
def test_merge_alias(requires):
    """
    basic lockfile merging including alias
    """
    c = TestClient()
    app = textwrap.dedent(f"""
        from conan import ConanFile
        class App(ConanFile):
            settings = "build_type"
            def requirements(self):
                if self.settings.build_type == "Debug":
                    self.{requires}("pkg/(alias_debug)")
                else:
                    self.{requires}("pkg/(alias_release)")
        """)
    c.save({"pkg/conanfile.py": GenConanfile("pkg"),
            "alias_release/conanfile.py": GenConanfile("pkg", "alias_release").with_class_attribute(
                "alias = 'pkg/0.1'"),
            "alias_debug/conanfile.py": GenConanfile("pkg", "alias_debug").with_class_attribute(
                "alias = 'pkg/0.2'"),
            "app/conanfile.py": app})
    c.run("create pkg --version=0.1")
    c.run("create pkg --version=0.2")
    c.run("export alias_release")
    c.run("export alias_debug")
    c.run("lock create app -s build_type=Release --lockfile-out=release.lock")
    c.run("lock create app -s build_type=Debug --lockfile-out=debug.lock")

    c.run("lock merge --lockfile=release.lock --lockfile=debug.lock --lockfile-out=conan.lock")

    # Update alias, won't be used
    c.save({"alias_release/conanfile.py": GenConanfile("pkg", "alias_release").with_class_attribute(
                "alias = 'pkg/0.3'"),
            "alias_debug/conanfile.py": GenConanfile("pkg", "alias_debug").with_class_attribute(
                "alias = 'pkg/0.4'")})
    c.run("export alias_release")
    c.run("export alias_debug")

    # Merged one can resolve both aliased without issues
    c.run("install app -s build_type=Release --lockfile=conan.lock")
    is_build_requires = requires == "tool_requires"
    c.assert_listed_require({"pkg/0.1": "Cache"}, build=is_build_requires)
    c.run("install app -s build_type=Debug --lockfile=conan.lock")
    c.assert_listed_require({"pkg/0.2": "Cache"}, build=is_build_requires)

    # without lockfiles it would be pointing to the new (unexistent) ones
    c.run("install app -s build_type=Release", assert_error=True)
    assert "ERROR: Package 'pkg/0.3' not resolved" in c.out
    c.run("install app -s build_type=Debug", assert_error=True)
    assert "ERROR: Package 'pkg/0.4' not resolved" in c.out
